Package: oncoscanR
Type: Package
Title: Secondary analyses of CNV data (HRD and more)
Version: 1.12.0
Authors@R: c(person("Yann", "Christinat", email = "yann.christinat@hcuge.ch", 
                  role = c("aut", "cre")),
             person("Geneva University Hospitals", role=c("aut", "cph")))
Description: The software uses the copy number segments from a text file and 
    identifies all chromosome arms that are globally altered and computes various 
    genome-wide scores. The following HRD scores (characteristic of BRCA-mutated cancers) 
    are included: LST, HR-LOH, nLST and gLOH. the package is tailored for the 
    ThermoFisher Oncoscan assay analyzed with their Chromosome Alteration Suite 
    (ChAS) but can be adapted to any input.
License: MIT + file LICENSE
Date: 2025-06-27
Encoding: UTF-8
VignetteBuilder: knitr
biocViews: CopyNumberVariation, Microarray, Software
Imports: readr, S4Vectors, methods, utils
RoxygenNote: 7.3.2
Depends: R (>= 4.2), IRanges (>= 2.30.0), GenomicRanges (>= 1.48.0),
        magrittr
Suggests: testthat (>= 3.1.4), jsonlite, knitr, rmarkdown, BiocStyle
URL: https://github.com/yannchristinat/oncoscanR
BugReports: https://github.com/yannchristinat/oncoscanR/issues
LazyData: true
git_url: https://git.bioconductor.org/packages/oncoscanR
git_branch: RELEASE_3_22
git_last_commit: 11be7a3
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:32:25 UTC; biocbuild
Author: Yann Christinat [aut, cre],
  Geneva University Hospitals [aut, cph]
Maintainer: Yann Christinat <yann.christinat@hcuge.ch>

# Generated by roxygen2: do not edit by hand

export(adjust_loh)
export(armlevel_alt)
export(get_amp_segments)
export(get_gain_segments)
export(get_hetloss_segments)
export(get_homloss_segments)
export(get_loh_segments)
export(get_loss_segments)
export(get_oncoscan_coverage_from_bed)
export(load_ascat)
export(load_chas)
export(merge_segments)
export(prune_by_size)
export(score_avgcn)
export(score_estwgd)
export(score_gloh)
export(score_loh)
export(score_lst)
export(score_mbalt)
export(score_nlst)
export(score_td)
export(trim_to_coverage)
export(workflow_oncoscan.ascat)
export(workflow_oncoscan.chas)
import(GenomicRanges)
import(IRanges)
import(S4Vectors)
import(readr)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(utils,read.table)

# oncoscanR
author: Yann Christinat

date: April 26, 2023

version: 1.7.0 (bioconductor-devel)

## Description
OncoscanR is an R package to handle Copy Number Variation analyses originating 
from the Applied Biosystems™️ OncoScan™️ CNV Assay. It allows computation of 
different homologous recombination deficiency (HRD) scores[Telli 2016], including the nLST 
score used by the Geneva HRD test[Christinat 2023], and the tandem 
duplication plus score (TDplus) to identify CDK12-mutated tumors [Popova 2016]. 

The package also allows for identification of arm-level alterations (e.g. gain 
of chromosome arm 1p).

**IMPORTANT**: The package expects as input the text exported file from ChAS 
(Chromosome Analysis Suite; the Applied Biosystems™️ software to identify CNV 
segments from the OncoScan™️ CNV Assay). The package assumes that all segments 
given in the file are correct and true. The ChAS text file has to contain the 
columns `Type`, `CN State` and `Full Location` (to setup in ChAS). Any text file 
that complies with this structure should work equally well.

Starting with version 1.2.0 (github), ASCAT output files can also be used as input.

Note that the Oncoscan does not cover the p arms of chromosome 13, 14, 15 and 22. 
The coverage on the p arm of chromosome 21 is only partial and is not included 
in this package.


**Disclaimer**: The use of OncoScan™️ platform in this research
does not imply an endorsement or recommendation of Thermo
Fisher Scientific Inc. and its subsidiaries for the utilization of
any specific algorithm or methodology with the OncoScan™️
platform for HRD analysis. Thermo Fisher Scientific Inc. and
its subsidiaries make no claims regarding the suitability,
performance or efficacy of any algorithms or methodologies
used in conjunction with the OncoScan™️ platform for HRD
analysis. Furthermore, Thermo Fisher Scientific Inc. and its
subsidiaries take no responsibility, and anyone using any
specific algorithms or methodologies in conjunction with the
OncoScan™️ platform for HRD analysis is solely responsible
for researching, identifying, and obtaining any necessary third-party
rights and ensuring that such use is in compliance with
applicable laws and regulations.

### Computation of arm-level alteration
An arm is declared globally altered if more than 90% of its bases are altered with a similar CNV type (amplifications
[5 copies or more], gains [1-2 extra copies], losses or copy-neutral losses of heterozygozity [LOH])[Christinat 2021]. For
instance, "gain of 3p" indicates that there is more than 90% of arm with 3 copies but less than 90% with 5 (otherwise
it would be an amplification). Prior to computation, segments of same copy number and at a distance <300Kbp (Oncoscan
resolution genome-wide) are merged. The remaining segments are filtered to a minimum size of 300Kbp.


### HRD scores
#### Score nLST
HRD score developed at HUG and based on the LST score by Popova et al. but normalized by an estimation of the number of
whole-genome doubling events.Of note, copy-neutral LOH segments are removed before computation.

`nLST = LST - 7*W/2` where `W` is the number of whole-genome doubling events.

The score is positive if there are at least 15 nLST.

The nLST score has been validated on 469 high grade ovarian cancer samples from
the PAOLA-1 clinical trial and is used in routine at the Geneva University Hospitals 
for prediction of PARP inhibitors response.

*How to cite*

Yann Christinat et al., Normalized LST Is an Efficient Biomarker for Homologous Recombination Deficiency and Olaparib Response in Ovarian Carcinoma. *JCO Precis Oncol* 7, e2200555(2023).
DOI:10.1200/PO.22.00555


#### Score LST
Procedure based on the paper from Popova et al, Can. Res. 2012 (PMID: 22933060). First segments
smaller than 3Mb are removed, then segments are smoothed with respect to copy number at a distance of 3Mb.
The number of LSTs is the number of breakpoints (breakpoints closer than 3Mb are merged) that have a segment
larger or equal to 10Mb on each side. This score was linked to BRCA1/2-deficient tumors.

#### Score LOH
Procedure based on the paper from Abkevich et al., Br J Cancer 2012 (PMID: 23047548). 
Number of LOH segments larger than 15Mb but excluding segments on chromosomes with a global LOH alteration. 
This score was linked to BRCA1/2-deficient tumors.

#### Score gLOH
The percentage genomic LOH score is computed as described in the FoundationFocus CDx BRCA
LOH assay; i.e. the percentage of bases covered by the Oncoscan that display a loss of heterozygosity
independently of the number of copies, excluding chromosomal arms that have a global LOH
(>=90% arm length). To compute with the armlevel_alt function on LOH segments only). This
score was linked to BRCA1/2-deficient tumors.

### Score TDplus
Procedure based on the paper from Popova et al., Cancer Res 2016 (PMID: 26787835). The TDplus
score is defined as the number of regions larger than 1Mb but smaller or equal to 10Mb with a gain of one
or two copies. This score was linked to CDK12-deficient tumors. 
They also identified as second category of tandem duplication whose size is smaller or equal than 1Mb and around 
300Kb but could not link it to a phenotype. Note that due to its resolution the Oncoscan assay will most likely miss 
this second category. Nonetheless it is reported by the function but not by the standard workflow.

## Installation
The package requires the prior installation of the packages `GenomicRanges` (bioconductor), `magrittr`, `jsonlite` and
`readr`.

```{r}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("GenomicRanges")

install.packages(c("magrittr", "jsonlite", "readr"))
```

There are three options to install the oncoscanR package: 

1. Install via bioconductor (nightly build):
```{r}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

# The following initializes usage of Bioc devel
BiocManager::install(version='devel')

BiocManager::install("oncoscanR")
```
2. Install from tarball. Download the `oncoscanR_1.2.0.tar.gz` file (stable version). Then in R, set the working directory to where the
compressed package is and run:
```{r}
install.packages('oncoscanR_1.2.0.tar.gz', repos=NULL, type='source')
```
3. Install from GitHub. In R, install the devtools package (`install.packages('devtools')`), then run:
```{r}
library(devtools)
install_github('yannchristinat/oncoscanR')
```


## Testing the installation
Open R and type the following commands:
```{r}
library(oncoscanR)
segs.filename <- system.file("extdata", "chas_example.txt", package = "oncoscanR")
workflow_oncoscan.chas(segs.filename)
```

If everything is setup fine, it should return a list with no arm-level alterations and a negative HRD score (nLST=1).


## Usage
The main workflow can be launched either in R via the `workflow_oncoscan.chas(chas.fn)` function or via the
script `bin/run_oncoscan_workflow.R`:

Usage: `Rscript path_to_oncoscanR_package/scripts/run_oncoscan-workflow.R CHAS_FILE`
- `CHAS_FILE`: Path to the text export file from ChAS or a compatible text file.

The script will output a JSON string into the terminal with all the computed information. :

```{json}
{
  "armlevel": {
    "AMP": [],
    "LOSS": ["17p", "2q", "4p"],
    "LOH": ["14q", "5q", "8p", "8q"],
    "GAIN": [19p", "19q", "1q", "20p", "20q", "3q", "5p", "6p", "9p", "9q", "Xp", "Xq"]
  },
  "scores": {
    "HRD": "Negative, nLST=12",
    "TDplus": 22,
    "avgCN": "2.43"
  }
  "file": "path/to/original_ChAS_file.txt"
}
```

Please read the vignette for more details and the manual for a description of all available R functions.

## References
- Abkevich et al., Patterns of genomic loss of heterozygosity predict homologous recombination repair defects in epithelial ovarian cancer, Br J Cancer, 107(10), 2012.
- Carter et al., Absolute quantification of somatic DNA alterations in human cancer, Nat Biotech, volume 30(5), 2012.
- Christinat et al., Automated Detection of Arm-Level Alterations for Individual Cancer Patients in the Clinical Setting, J Mol Diagn, 23(17):1722-1731, 2021.
- Christinat et al., Normalized LST Is an Efficient Biomarker for Homologous Recombination Deficiency and Olaparib Response in Ovarian Carcinoma, JCO Precision Oncology, volume 7, 2023.
- Popova et al., Ploidy and large-scale genomic instability consistently identify basal-like breast carcinomas with BRCA1/2 inactivation, Cancer Res volume 72(21), 2012.
- Popova et al., Ovarian Cancers Harboring Inactivating Mutations in CDK12 Display a Distinct Genomic Instability Pattern Characterized by Large Tandem Duplications, Cancer Res volume 76(7), 2016.
- Telli et al., Homologous Recombination Deficiency (HRD) Score Predicts Response to Platinum-Containing Neoadjuvant Chemotherapy in Patients with Triple-Negative Breast Cancer, Clin Cancer Res volume 22(15), 2016.

# File `chas_example.txt`
Toy example containing a subset of the alterations of a real ChAS export file.

# File `ascat_example.txt`
Toy example containing a subset of the alterations of a ASCAT result file.

# File `OncoScan_CNV.na33.r2.annot.processed.bed`
Arm coverage of the Oncoscan na33.r2 assay. Of note, the arms 13p, 14p, 15p,
21p and 22p are not covered by the Oncoscan.

## Source
References downloaded from the ThermoFisher website:
OncoScan_CNV_Analysis_Files_NA33.r5.zip. Then the position of each probe was
extracted from the SQLite database (OncoScan_CNV.na33.r2.annot.db file) with
the following command:

`SELECT Chr_id, Start, Stop, Cytoband, dbSNP_RS_ID, ProbeSet_ID FROM
Annotations WHERE Process_Flag > 0;`

Data was then saved in a temporary file (`sql-output.csv`) and processed to 
group probes based on their distance to the next probe (different groups if 
more than 300Kbp, the genome-wide Oncoscan resolution). 
Groups with less than 10 probes and situated at the ends of the
chromosome or close to the centromere were discarded.

Grouping procedure in python3:

```{python}
snp = dict()
with open('sql-output.csv', 'r') as f:
    h = f.readline()  # skip headers
    for l in f:
        toks = l.strip().split('\t')
        arm = toks[0] + toks[3][0]
        pos = int(toks[1])
        if arm not in snp:
            snp[arm] = []
        snp[arm].append(pos)

for arm in sorted(snp.keys()):
    snps = sorted(snp[arm])
    start = {'pos': snps[0], 'i':0}
    for i in range(len(snps)-1):
        if snps[i+1]-snps[i] > 300000:
            print('\t'.join([arm, str(start['pos']), str(snps[i]),
                             str(i+1-start['i']),
                             str((snps[i]-start['pos'])/1000)]))
            start = {'pos': snps[i+1], 'i': i+1}
    print('\t'.join([arm, str(start['pos']), str(snps[-1]),
                     str(len(snps)+1-start['i']),
                     str((snps[-1]-start['pos'])/1000)]))
```

The output of the python script was saved in the file 
`OncoScan_CNV.na33.r2.annot.processed.bed`

## Format
A BED file with 3 columns (chromosome, start, end) and one line for each arm.

# File `LST_gene_list_full_location.txt`
A ChAS export file of a tumor sample with an HR-deficient phenotype

# File `TDplus_gene_list_full_location.txt`
A ChAS export file of a tumor sample with a CDK12-mutated phenotype

# File `triploide_gene_list_full_location.txt`
A ChAS export file of a sample with a triploid tumor

# scores.R Functions to identify the globally-altered arms and computes various
# scores.  Author: Yann Christinat Date: 19.9.2019


#' Get all globally-altered chromosome arms.
#'
#' @details By default uses the sum of all alterations and set the arm as
#' globally altered if \>80\% of the arm is altered. Does not account for
#' alteration type and copy number.
#' Will run the function \code{trim_to_coverage} on the segments.
#'
#' @param segments A \code{GRanges} object containing the segments.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#' @param threshold The minimum percentage of the arm to be considered as
#' globally altered. Defaults to 80\%.
#'
#' @return A list of globally-altered chromosome arms with the percentage of arm
#'  altered.
#' @export
#'
#' @import GenomicRanges
#' @importFrom methods is
#'
#' @examples
#' arms <- armlevel_alt(segs.chas_example, oncoscan_na33.cov, 0.9)
armlevel_alt <- function(segments, kit.coverage, threshold = 0.9) {
    is_cn_segment(segments, raise_error = TRUE)
    stopifnot(is(kit.coverage, "GRanges"))

    vals <- vapply(levels(seqnames(kit.coverage)), function(arm) {
        if (!(arm %in% as.vector(seqnames(segments)))) {
            return(0)
        }
        # Get the number of bases covered by the kit
        iscov <- as.vector(seqnames(kit.coverage)) == arm
        armlen <- sum(width(kit.coverage[iscov]))

        # Run trim_to_coverage on arm segments
        armsegs <- segments[as.vector(seqnames(segments)) == arm]
        segs <- trim_to_coverage(armsegs, kit.coverage)
        if (length(segs) == 0) {
            return(0)
        }


        # Set the copy number to NA to make sure overlapping segments will be
        # merged, then merge
        segs$cn <- NA
        segs.flat <- merge_segments(segs, 1 / 1000)

        # Get the number of bases altered
        segslen <- sum(width(segs.flat))

        return(segslen / armlen)
    }, c(0), USE.NAMES = TRUE)

    return(vals[vals >= threshold])
}


#' Compute the number of Large-scale State Transitions (LSTs).
#'
#' @details Procedure based on the paper from Popova et al, Can. Res. 2012
#' (PMID: 22933060). First segments smaller than 3Mb are removed, then segments
#' are smoothed with respect to copy number at a distance of 3Mb.
#' The number of LSTs is the number of breakpoints (breakpoints closer than 3Mb
#' are merged) that have a segment larger or equal to 10Mb on each side. This
#' score was linked to BRCA1/2-deficient tumors.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#'
#' @return An integer representing the number of LSTs.
#' @export
#'
#' @examples
#' score_lst(segs.chas_example, oncoscan_na33.cov)
score_lst <- function(segments, kit.coverage) {
    is_cn_segment(segments, raise_error = TRUE)
    stopifnot(is(kit.coverage, "GRanges"))

    # Prune all segments < 3Mb
    segs.min3mb <- prune_by_size(segments, 3000)
    if (length(segs.min3mb) == 0) {
        return(0)
    }

    # Merge segments by CN at a distance of 3Mb
    segs.merged <- merge_segments(segs.min3mb, 3000)

    # Compute the number of Large-scale State Transitions for each arm
    vals <- lapply(unique(seqnames(segs.merged)), function(arm) {
        arm.cov <- kit.coverage[as.vector(seqnames(kit.coverage)) == arm]
        segs <- segs.merged[as.vector(seqnames(segs.merged)) == arm]

        # Get breakpoints and start/end of arm
        starts <- start(segs)
        ends <- end(segs)
        breakpoints <-
            sort(unique(c(start(arm.cov), starts, ends + 1, end(arm.cov) + 1)))

        if (length(breakpoints) < 3) {
            # Since we add the start/end of arm, there has to be at least 3
            # elements in 'breakpoints' to have LSTs
            return(0)
        }

        # Smooth breakpoints at 3Mb
        for (i in 2:length(breakpoints)) {
            if (breakpoints[i] - breakpoints[i - 1] < 3*10^6) {
                # Merge breakpoints
                midpoint <- round((breakpoints[i] + breakpoints[i - 1]) / 2)
                breakpoints[i - 1] <- NA
                breakpoints[i] <- midpoint
            }
        }
        bp.smooth <- breakpoints[!is.na(breakpoints)]

        if (length(bp.smooth) < 3) {
            # Since we add the start/end of arm, there has to be at least 3
            # elements in 'breakpoints' to have LSTs
            return(0)
        }

        # Get the number of LSTs
        n.lst <- 0
        for (i in 2:(length(bp.smooth) - 1)) {
            prev.gt10m <- bp.smooth[i] - bp.smooth[i - 1] >= 10*10^6
            next.gt10m <- bp.smooth[i + 1] - bp.smooth[i] >= 10*10^6
            n.lst <- ifelse(prev.gt10m & next.gt10m, n.lst + 1, n.lst)
        }
        return(n.lst)
    })
    return(sum(unlist(vals)))
}


#' Compute the number HR deficiency-associated LOH regions.
#'
#' @details Procedure based on the paper from Abkevich et al., Br J Cancer 2012
#' (PMID: 23047548). All LOH segments larger than 15Mb but excluding chromosome
#' with a global LOH alteration (to compute with the \code{armlevel_alt}
#' function on LOH segments only). This score was linked to BRCA1/2-deficient
#' tumors.
#' Note that the function will merge overlapping or neighbor LOH segments (at a
#' distance of 1bp).
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param arms.loh A list of arms with global/arm-level LOH alteration.
#' @param arms.hetloss A list of arms with global/arm-level heterozygous
#' losses.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#'
#' @return An integer representing the number of HRD-LOH regions.
#' @export
#'
#' @examples
#' armlevel.loh <- armlevel_alt(get_loh_segments(segs.chas_example),
#'                              kit.coverage = oncoscan_na33.cov)
#' armlevel.hetloss <- armlevel_alt(get_hetloss_segments(segs.chas_example),
#'                              kit.coverage = oncoscan_na33.cov)
#' score_loh(segs.chas_example, names(armlevel.loh), names(armlevel.hetloss),
#' oncoscan_na33.cov)
score_loh <- function(segments, arms.loh, arms.hetloss, kit.coverage) {
    is_cn_segment(segments, raise_error = TRUE)
    stopifnot(is(kit.coverage, "GRanges"))

    if (length(segments) == 0) { return(0) }

    # Test if any arms in arms.loh or arms.loss is not in kit.coverage
    if (length(c(arms.loh, arms.hetloss)) > 0) {
        unknown.arms <- setdiff(c(arms.loh, arms.hetloss),
                                seqnames(kit.coverage))
        if (length(unknown.arms) > 0) {
            msg <- paste('Some arms were not found in the kit:',
                         paste(as.character(unknown.arms), collapse = TRUE))
            stop(msg)
        }
    }

    arms.to_ban <- c()
    kit.chroms <- as.character(seqnames(kit.coverage))
    chromnames <- unique(unlist(strsplit(kit.chroms, "[pq]")))

    # Get chromosomes with LOH on both arms
    if (length(arms.loh) > 0) {
        # Check on each chromosome if both arms are within the parameter
        # 'arms.loh'. Also account for the fact that a chromosome may have
        # only one arm covered by the kit.
        arms_found.to_ban <- lapply(chromnames, is_arm_to_ban,
                                    arms=arms.loh, kit=kit.coverage)
        arms.to_ban <- append(arms.to_ban, do.call("c", arms_found.to_ban))
    }
    # Get chromosomes with het loss on both arms
    if (length(arms.hetloss) > 0) {
        # Check on each chromosome if both arms are within the parameter
        # 'arms.loh'. Also account for the fact that a chromosome may have
        # only one arm covered by the kit.
        arms_found.to_ban <- lapply(chromnames, is_arm_to_ban,
                                    arms=arms.hetloss, kit=kit.coverage)
        arms.to_ban <- append(arms.to_ban, do.call("c", arms_found.to_ban))
    }
    # Get LOH segments larger than 15Mb and not in 'arms.to_ban'
    segs.loh <- c(get_loh_segments(segments), get_hetloss_segments(segments))
    segs.loh <- segs.loh[!(as.vector(seqnames(segs.loh)) %in% arms.to_ban)]
    segs.merged <- merge_segments(segs.loh, 1 / 1000)
    segs.loh_15m <- segs.merged[width(segs.merged) > 15*10^6]
    return(length(segs.loh_15m))
}


#' Helper function to detect if a chromosome is within a set of chromosomal arms
#'
#' @details Used in the score_loh function
#'
#' @param chrom String of a chromosome
#' @param arms list of chromosomal arms (strings)
#' @param kit A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#'
#' @return A list of chromosomal arms (string)
#'
#' @noRd
is_arm_to_ban <- function(chrom, arms, kit) {
    chrom.arms <- paste0(chrom, c("p", "q"))

    arms.covered <- intersect(chrom.arms, as.vector(seqnames(kit)))

    arms.found <- unique(intersect(arms.covered, arms))
    if (length(arms.found) == length(arms.covered)) {
        return(as.character(arms.covered))
    }
}


#' Compute the number of large tandem duplication (TDplus).
#'
#' @details Procedure based on the paper from Popova et al., Cancer Res 2016
#' (PMID: 26787835). The TDplus score is defined as the number of regions larger
#'  than 1Mb but smaller or equal to 10Mb with a gain of one or two copies. This
#'  score was linked
#'  to CDK12-deficient tumors. They also identified as second category of tandem
#'   duplication whose size is smaller or equal than 1Mb and around 300Kb but
#'   could not link it to a phenotype. Note that due to its resolution the
#'   Oncoscan assay will most likely miss this second category. Nonetheless it
#'   is reported by the function.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#'
#' @return A list of integer containing the TDplus score (\code{'TDplus'}) and
#' the small TD score (\code{'TD'}).
#' @export
#'
#' @examples
#' score_td(segs.chas_example)
score_td <- function(segments) {
    is_cn_segment(segments, raise_error = TRUE)

    if (length(segments) == 0) {
        return(list(TDplus = 0, TD = 0))
    }


    segs.gain <- segments[segments$cn.type == cntypes$Gain & segments$cn <= 4]
    segs.width <- width(segs.gain)
    segs.tdplus <- segs.gain[segs.width > 1*10^6 & segs.width <= 10*10^6]
    segs.td <- segs.gain[segs.width <= 1*10^6]

    return(list(TDplus = length(segs.tdplus), TD = length(segs.td)))
}



#' Compute the average copy number variation across the genome.
#'
#' @details Compute the weighted average (by segment length) of the copy number
#' variation. LOH segments and sexual chromosomes are excluded. Copy number
#' variation is rounded to the next level (1.67 -> 1 but 2.33 -> 3).
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#'
#' @return A decimal value
#' @export
#'
#' @examples
#' score_avgcn(segs.chas_example, oncoscan_na33.cov)
score_avgcn <- function(segments, kit.coverage) {
    is_cn_segment(segments, raise_error = TRUE)
    stopifnot(is(kit.coverage, "GRanges"))

    autosomes <-
        as.vector(seqnames(kit.coverage)[!(as.vector(seqnames(kit.coverage))
                                           %in% c("Xp", "Xq", "Yp", "Yq"))])

    # Total number of bases (in Mbp) covered by the kit
    kitwidth.noXY <-
        sum(width(kit.coverage[as.vector(seqnames(kit.coverage)) %in%
                                   autosomes]) / 10^6)

    # Select segments on autosomes and with non-copy neutral alterations
    segs <-
        segments[as.vector(seqnames(segments)) %in% autosomes &
                     segments$cn.type %in%
                     c(cntypes$Gain, cntypes$Loss)]
    if (length(segs) == 0) {
        return(2)
    }

    segs.w <- width(segs) / 10^6  # Width of each segment in Mbp

    # Round the copy number in case of subclones
    cn <- segs$cn
    cn[cn < 2] <- floor(cn[cn < 2])
    cn[cn > 2] <- ceiling(cn[cn > 2])

    avgcn <-
        (sum(segs.w * cn) + 2 * (kitwidth.noXY - sum(segs.w))) / kitwidth.noXY
    return(avgcn)
}


#' Estimates the number of whole-genome doubling events (WGD).
#'
#' @details Based on the publication from Carter et al. (Nature Biotechnology
#' 2012; PubMed ID: 22544022).
#' On a pan-cancer cohort, they observed that tumors that underwent one
#' whole-genome doubling event had a ploidy (average copy number) between 2.2
#' and 3.4. This function relies on the function \code{score_avgcn} to compute
#' the ploidy.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#'
#' @return A named list with two values: WGD (whole-genome doubling events) and
#' avgCN (the average copy number). WGD values are 0 for no WGD event, 1 for one
#'  WGD event, 2 for several WGD events.
#' @export
#'
#' @examples
#' score_estwgd(segs.chas_example, oncoscan_na33.cov)
score_estwgd <- function(segments, kit.coverage) {
    is_cn_segment(segments, raise_error = TRUE)
    stopifnot(is(kit.coverage, "GRanges"))

    # Get the average copy number
    avgcn <- score_avgcn(segments, kit.coverage)

    # Estimate the number of WGD events
    wgd.est <- ifelse(avgcn > 3.4, 2, ifelse(avgcn > 2.2, 1, 0))

    return(c(WGD = wgd.est, avgCN = avgcn))
}


#' Compute the number of LSTs, normalized by the number of WGD events.
#'
#' @details Compute the number of LSTs in non-LOH segments via the
#' \code{score_lst} function and subtract the extra noise induced by WGD events:
#'  nLST = LST - 7*W/2 where W is the number of WGD events.
#' A sample is HRD positive (deficient in HR pathway) if nLST is greater or
#' equal to the threshold (15 by default).
#' This score was linked to BRCA1/2-deficient tumors.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param n.wgd Number of whole-genome doubling events (0 if diploid).
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#' @param threshold A number above which the test is returned positive (>=).
#'
#' @return A named list with the number of nLSTs and the corresponding label
#' ('Positive', 'Negative').
#' @export
#'
#' @examples
#' w <- score_estwgd(segs.chas_example, oncoscan_na33.cov)
#' score_nlst(segs.chas_example, w['WGD'], oncoscan_na33.cov)
score_nlst <-
    function(segments, n.wgd, kit.coverage, threshold = 15) {
        is_cn_segment(segments, raise_error = TRUE)
        stopifnot(is(kit.coverage, "GRanges"))

        lst.noLOH <-
            score_lst(segments[segments$cn.type != cntypes$LOH], kit.coverage)

        nlst <- max(0, lst.noLOH - 3.5 * as.numeric(n.wgd))
        label <- ifelse(nlst >= threshold, "Positive", "Negative")
        return(c(nLST = nlst, HRD = label))
    }


#' Computes the total number of Mbp altered.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#' @param loh.rm A boolean (TRUE by default) to indicate whether LOH segments
#' should be excluded.
#'
#' @return A named list representing the Mbp altered in the sample and the total
#'  Mbp of the kit.
#' @export
#'
#' @examples
#' score_mbalt(segs.chas_example, oncoscan_na33.cov)
#' score_mbalt(segs.chas_example, oncoscan_na33.cov, FALSE)
score_mbalt <- function(segments, kit.coverage, loh.rm = TRUE) {
    is_cn_segment(segments, raise_error = TRUE)
    stopifnot(is(kit.coverage, "GRanges"))

    # Compute the number Mbp present in the whole kit
    mb.kit <- round(sum(width(kit.coverage) / 10^6))

    # Exclude (or not) the LOH segments
    segs <- segments
    if (loh.rm) {
        segs <- segments[segments$cn.type != cntypes$LOH]
    }

    if (length(segs) == 0) {
        return(c(sample = 0, kit = mb.kit))
    } else {
        segs.w <- width(reduce(segs)) / 10^6  # Width of each segment in Mbp
        mb.alt <- round(sum(segs.w))

        return(c(sample = mb.alt, kit = mb.kit))
    }
}


#' Compute the genomic LOH score.
#'
#' @details The percentage genomic LOH score is computed as described in the
#' FoundationFocus CDx BRCA LOH assay; i.e. the percentage of bases covered by
#' the Oncoscan that display a loss of heterozygosity independently of the
#' number of copies, excluding chromosomal arms that have a global LOH (>=90% of
#' arm length).
#' To compute with the \code{armlevel_alt} function on LOH segments only).
#' This score was linked to BRCA1/2-deficient tumors.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param arms.loh A list of arms with global/arm-level LOH alteration.
#' @param arms.hetloss A list of arms with global/arm-level heterozygous
#' loss.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#'
#' @return An integer representing the percentage of LOH bases.
#' @export
#'
#' @examples
#' armlevel.loh <- armlevel_alt(get_loh_segments(segs.chas_example),
#'                              kit.coverage = oncoscan_na33.cov)
#' armlevel.hetloss <- armlevel_alt(get_hetloss_segments(segs.chas_example),
#'                              kit.coverage = oncoscan_na33.cov)
#' score_gloh(segs.chas_example, names(armlevel.loh), names(armlevel.hetloss),
#' oncoscan_na33.cov)
score_gloh <- function(segments, arms.loh, arms.hetloss, kit.coverage) {
    is_cn_segment(segments, raise_error = TRUE)
    stopifnot(is(kit.coverage, "GRanges"))

    # Test if any arms in arms.loh or arms.loss is not in kit.coverage
    if (length(c(arms.loh, arms.hetloss)) > 0) {
        unknown.arms <- setdiff(c(arms.loh, arms.hetloss),
                                seqnames(kit.coverage))
        if (length(unknown.arms) > 0) {
            msg <- paste('Some arms were not found in the kit:',
                         paste(as.character(unknown.arms), collapse = TRUE))
            stop(msg)
        }
    }

    arms.to_ban <- c(arms.loh, arms.hetloss)

    segs.loh <- c(get_loh_segments(segments), get_hetloss_segments(segments))
    segs.loh <- segs.loh[!(as.vector(seqnames(segs.loh)) %in% arms.to_ban)]

    width.loh <- width(segs.loh)
    return(sum(width.loh) / sum(width(kit.coverage)))
}

#' On a simple case with 4 chromosomes and 6 alterations (including an overlapping LOH segment)
#' TOT bp = 4*100 = 400
#' ALT bp weigthed by CN = 3*20 + 1*20 + 0*10 + 14*50 = 780
#' NON-ALT bp weigthed by CN = 2*(TOT bp - (20+20+10+50)) = 2*300 = 600
#' avg CN = (780+600)/400 = 3.45
test_that("Avg CN computes correctly", {
    cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                   ranges = IRanges(start = rep(1, 4),
                                    end = rep(100, 4)))
    segs <- GRanges(seqnames = factor(c(rep('1p', 4), rep('2p', 2)), levels = paste0(1:4, 'p')),
                    ranges = IRanges(start = c(30, 50, 70,  90, 25,  50) + 1,
                                     end =   c(50, 70, 90, 100, 75, 100)),
                    cn = c(NA, 3, 1, 0, NA, 14),
                    cn.type = c("LOH", "Gain", "Loss", "Loss",
                                "LOH", "Gain"))
    avgcn <- score_avgcn(segs, cov)
    expect_equal(avgcn, 3.45)
})

#' Test the limits at 0, 2, 2.19, 2.2, 2.21, 3.38, 3.4, 3.42 and 10
test_that("Est WGD works on limit cases", {
    cov <- GRanges(seqnames = factor('1p', levels = c('1p')),
                   ranges = IRanges(start = 1, end = 100))
    # Test at avgCN=0
    seg <- GRanges(seqnames = factor('1p', levels = c('1p')),
                ranges = IRanges(start = 1, end = 100), cn=0, cn.type = "Loss")
    est <- score_estwgd(seg, cov)
    expect_equal(est, c(WGD=0,avgCN=0))

    # Test at avgCN=2 (no segments)
    est <- score_estwgd(GRanges(), cov)
    expect_equal(est, c(WGD=0,avgCN=2))

    # Test at avgCN=2.19
    seg <- GRanges(seqnames = factor('1p', levels = c('1p')),
                   ranges = IRanges(start = 1, end = 19), cn=3, cn.type = "Gain")
    est <- score_estwgd(seg, cov)
    expect_equal(est, c(WGD=0,avgCN=2.19))

    # Test at avgCN=2.2
    seg <- GRanges(seqnames = factor('1p', levels = c('1p')),
                   ranges = IRanges(start = 1, end = 20), cn=3, cn.type = "Gain")
    est <- score_estwgd(seg, cov)
    expect_equal(est, c(WGD=0,avgCN=2.2))

    # Test at avgCN=2.21
    seg <- GRanges(seqnames = factor('1p', levels = c('1p')),
                   ranges = IRanges(start = 1, end = 21), cn=3, cn.type = "Gain")
    est <- score_estwgd(seg, cov)
    expect_equal(est, c(WGD=1,avgCN=2.21))

    # Test at avgCN=3.38
    seg <- GRanges(seqnames = factor('1p', levels = c('1p')),
                   ranges = IRanges(start = 1, end = 69), cn=4, cn.type = "Gain")
    est <- score_estwgd(seg, cov)
    expect_equal(est, c(WGD=1,avgCN=3.38))

    # Test at avgCN=3.4
    seg <- GRanges(seqnames = factor('1p', levels = c('1p')),
                   ranges = IRanges(start = 1, end = 70), cn=4, cn.type = "Gain")
    est <- score_estwgd(seg, cov)
    expect_equal(est, c(WGD=1,avgCN=3.4))

    # Test at avgCN=3.42
    seg <- GRanges(seqnames = factor('1p', levels = c('1p')),
                   ranges = IRanges(start = 1, end = 71), cn=4, cn.type = "Gain")
    est <- score_estwgd(seg, cov)
    expect_equal(est, c(WGD=2,avgCN=3.42))

    # Test at avgCN=10
    seg <- GRanges(seqnames = factor('1p', levels = c('1p')),
                   ranges = IRanges(start = 1, end = 100), cn=10, cn.type = "Gain")
    est <- score_estwgd(seg, cov)
    expect_equal(est, c(WGD=2,avgCN=10))
})


test_that("nLST works - simple case and limit cases", {
    cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                   ranges = IRanges(start = rep(1, 4),
                                    end = rep(100*10^6, 4)))
    segs <- GRanges(seqnames = factor(c(rep('1p', 4), rep('2p', 2)), levels = paste0(1:4, 'p')),
                    ranges = IRanges(start = c(30, 50, 70,  90, 25,  50)*10^6 + 1,
                                     end =   c(50, 70, 90, 100, 75, 100)*10^6),
                    cn = c(NA, 3, 1, 0, NA, 14),
                    cn.type = c("LOH", "Gain", "Loss", "Loss", "LOH", "Gain"))

    # Test with WGD=0, threshold=15 (default)
    n <- score_nlst(segs, 0, cov)
    expect_equal(n, c(nLST=4, HRD='Negative'))

    # Test with WGD=0, threshold=4
    n <- score_nlst(segs, 0, cov, 4)
    expect_equal(n, c(nLST=4, HRD='Positive'))

    # Test with WGD=0, threshold=3.5
    n <- score_nlst(segs, 0, cov, 3.5)
    expect_equal(n, c(nLST=4, HRD='Positive'))

    # Test with WGD=1, threshold=15 (default)
    n <- score_nlst(segs, 1, cov)
    expect_equal(n, c(nLST=0.5, HRD='Negative'))

    # Test with WGD=2, threshold=15 (default)
    n <- score_nlst(segs, 2, cov)
    expect_equal(n, c(nLST=0, HRD='Negative'))

    # Test with WGD=3, threshold=15 (default)
    n <- score_nlst(segs, 3, cov)
    expect_equal(n, c(nLST=0, HRD='Negative'))

})

test_that("Mbp alt works", {
    cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                   ranges = IRanges(start = rep(1, 4),
                                    end = rep(100*10^6, 4)))
    segs <- GRanges(seqnames = factor(c(rep('1p', 4), rep('2p', 2)), levels = paste0(1:4, 'p')),
                    ranges = IRanges(start = c(30, 50, 70,  90, 25,  50)*10^6 + 1,
                                     end =   c(50, 70, 90, 100, 75, 100)*10^6),
                    cn = c(NA, 3, 1, 0, NA, 14),
                    cn.type = c("LOH", "Gain", "Loss", "Loss",
                                "LOH", "Gain"))

    mbalt.wLOH <- score_mbalt(segs, cov, loh.rm = FALSE)
    expect_equal(mbalt.wLOH, c(sample=145, kit=400))

    mbalt.noLOH <- score_mbalt(segs, cov, loh.rm = TRUE)
    expect_equal(mbalt.noLOH, c(sample=100, kit=400))

})

#' Test with no segments, 1 segment >80%, 1 segment <80%, 1 segment at 80%, 1 segment at 100%,
#' one arm with 3 segments and >80% in total, one arm with 3 segments and <80% in total.
test_that("Arm-level alteration detection works - basic", {
  cov <- GRanges(seqnames = factor(paste0(1:7, 'p'), levels = paste0(1:7, 'p')),
                 ranges = IRanges(start = rep(1, 7),
                                  end = rep(100, 7)))
  segs <- GRanges(seqnames = factor(c('2p','3p','4p','5p','6p','6p','6p','7p','7p','7p'),
                                    levels = paste0(1:7, 'p')),
                  ranges = IRanges(start = c(6,  21,  1,   1,  1, 21,  61,  1, 31, 91),
                                   end   = c(95, 80, 80, 100, 20, 50, 100, 10, 40, 100)),
                  cn = c(0,1,NA,3,5,10,0,1,NA,3),
                  cn.type = c("Loss", "Loss", "LOH", "Gain",
                              "Gain", "Gain", "Loss", "Loss",
                              "LOH", "Gain"))
  arms <- armlevel_alt(segs, cov, threshold = 0.8)
  expected <- c('2p'=0.9, '4p'=0.8, '5p'=1, '6p'=0.9)
  expect_equal(arms, expected)
})


test_that("Arm-level alteration detection works - 0% threshold", {
  cov <- GRanges(seqnames = factor(paste0(1:7, 'p'), levels = paste0(1:7, 'p')),
                 ranges = IRanges(start = rep(1, 7),
                                  end = rep(100, 7)))
  segs <- GRanges(seqnames = factor(c('2p','3p','4p','5p','6p','6p','6p','7p','7p','7p'),
                                    levels = paste0(1:7, 'p')),
                  ranges = IRanges(start = c(6,  21,  1,   1,  1, 21,  61,  1, 31, 91),
                                   end   = c(95, 80, 80, 100, 20, 50, 100, 10, 40, 100)),
                  cn = c(0,1,NA,3,5,10,0,1,NA,3),
                  cn.type = c("Loss", "Loss", "LOH", "Gain",
                              "Gain", "Gain", "Loss", "Loss",
                              "LOH", "Gain"))
  arms <- armlevel_alt(segs, cov, 0)
  expected <- c('1p'=0, '2p'=0.9, '3p'=0.6, '4p'=0.8, '5p'=1, '6p'=0.9, '7p'=0.3)
  expect_equal(arms, expected)
})


test_that("Arm-level alteration detection works - 100% threshold", {
  cov <- GRanges(seqnames = factor(paste0(1:7, 'p'), levels = paste0(1:7, 'p')),
                 ranges = IRanges(start = rep(1, 7),
                                  end = rep(100, 7)))
  segs <- GRanges(seqnames = factor(c('2p','3p','4p','5p','6p','6p','6p','7p','7p','7p'),
                                    levels = paste0(1:7, 'p')),
                  ranges = IRanges(start = c(6,  21,  1,   1,  1, 21,  61,  1, 31, 91),
                                   end   = c(95, 80, 80, 100, 20, 50, 100, 10, 40, 100)),
                  cn = c(0,1,NA,3,5,10,0,1,NA,3),
                  cn.type = c("Loss", "Loss", "LOH", "Gain",
                              "Gain", "Gain", "Loss", "Loss",
                              "LOH", "Gain"))
  arms <- armlevel_alt(segs, cov, 1)
  expected <- c('5p'=1)
  expect_equal(arms, expected)
})


test_that("Arm-level alteration detection works - overlaps", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = c(1,1,101,101),
                                  end = c(100,100,200,200)))
  segs <- GRanges(seqnames = factor(c(rep('1p',3), rep('2p',3), rep('3p',3), rep('4p',3)),
                                    levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c( 1, 16, 11,  1, 16, 11, 51,  99, 181, 51,  99, 171),
                                   end   = c(20, 50, 70, 20, 50, 80, 90, 110, 210, 90, 150, 210)),
                  cn = rep(c(0,6,NA), 4),
                  cn.type = rep(c("Loss", "Gain", "LOH"), 4))
  arms <- armlevel_alt(segs, cov, 0)
  expected <- c('1p'=0.7, '2p'=0.8, '3p'=0.3, '4p'=0.8)
  expect_equal(arms, expected)
})

test_that("Arm-level alteration detection works - empty segments", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = c(1,1,101,101),
                                  end = c(100,100,200,200)))
  segs <- GRanges(seqnames = factor(c(), levels = paste0(1:4, 'p')),
                  ranges = IRanges())
  arms <- armlevel_alt(segs, cov)
  expect_equal(length(arms), 0)
})

# utils.R Functions that are too general to fit in any other R file.
# Author: Yann Christinat Date: 24.9.2019


#' Test if the object is a CNV segment (GRange object with metadata: cn and 
#' cntype)
#'
#' @param obj Object to test
#' @param raise_error Boolean. If TRUE then raises an error if the test fails.
#'
#' @return Boolean. TRUE if the object is of class \code{GRanges} with fields
#' \code{cn} and \code{cn.type}.
#'
#' @noRd
#'
#' @importFrom methods is
is_cn_segment <- function(obj, raise_error = TRUE) {
    test <- is(obj, "GRanges") && (length(obj) == 0 || (!is.null(obj$cn) &&
        !is.null(obj$cn.type)))
    if (!test && raise_error) {
        stop("The parameter has to be a CNV segment (GRanges object with 'cn' 
             and 'cntype' columns).")
    }
    return(test)
}

#' oncoscanR: an R package to perform secondary analysis of Copy Number 
#' Variation data. 
#' 
#' Allows computation of different homologous recombination deficiency (HRD) 
#' scores to identify PARP inhibitors responders. The package also allows for 
#' computation of the tandem duplication plus score (TDplus; hallmark of 
#' CDK12-mutated tumors) and the identification of arm-level alterations (e.g. 
#' gain of chromosome arm 1p).
#' 
#' The package is tailored for the ThermoFisher Oncoscan assay analyzed with 
#' their Chromosome Alteration Suite (ChAS) but can be adapted to any input.
#' 
#' To learn more about \code{oncoscanR}, see the vignette using 
#' \code{browseVignettes(package = "oncoscanR")}.
#' 
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("oncoscanR")

## ----eval=FALSE---------------------------------------------------------------
# # install.packages('devtools')
# install_github('yannchristinat/oncoscanR')

## ----eval=FALSE---------------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("GenomicRanges")
# 
# install.packages(c("magrittr", "jsonlite", "readr"))

## -----------------------------------------------------------------------------
library(oncoscanR)
segs.filename <- system.file("extdata", "chas_example.txt", 
                             package = "oncoscanR")
res <- workflow_oncoscan.chas(segs.filename)
print(res)

## -----------------------------------------------------------------------------
library(magrittr)

# Load the ChAS file
chas.fn <- system.file("extdata", "chas_example.txt", package = "oncoscanR")
segments <- load_chas(chas.fn, oncoscan_na33.cov)

# Clean the segments: restricted to Oncoscan coverage, LOH not overlapping
# with copy loss segments, smooth&merge segments within 300kb and prune
# segments smaller than 300kb.
segs.clean <- trim_to_coverage(segments, oncoscan_na33.cov) %>%
    adjust_loh() %>%
    merge_segments() %>%
    prune_by_size()

## -----------------------------------------------------------------------------
ascat.fn <- system.file("extdata", "ascat_example.txt", package = "oncoscanR")
ascat.segments <- load_ascat(ascat.fn, oncoscan_na33.cov)
head(ascat.segments)

## -----------------------------------------------------------------------------
chas.fn <- system.file("extdata", "triploide_gene_list_full_location.txt", 
                       package = "oncoscanR")
segments <- load_chas(chas.fn, oncoscan_na33.cov)
armlevel.loh <- get_loh_segments(segments) %>% 
    armlevel_alt(kit.coverage = oncoscan_na33.cov)

## -----------------------------------------------------------------------------
armlevel.loh <- get_loh_segments(segments) %>%
        armlevel_alt(kit.coverage = oncoscan_na33.cov, threshold = 0)

## -----------------------------------------------------------------------------
mbalt <- score_mbalt(segments, oncoscan_na33.cov)
percent.alt <- mbalt['sample']/mbalt['kit']
message(paste(mbalt['sample'], 'Mbp altered ->', round(percent.alt*100), 
              '% of genome'))

avgcn <- score_avgcn(segments, oncoscan_na33.cov)
wgd <- score_estwgd(segments, oncoscan_na33.cov)
message(paste('Average copy number:', round(avgcn, 2), '->', wgd['WGD'], 
'whole-genome doubling event'))

## -----------------------------------------------------------------------------
# Load data
chas.fn <- system.file("extdata", "LST_gene_list_full_location.txt", 
                       package = "oncoscanR")
segments <- load_chas(chas.fn, oncoscan_na33.cov)

# Clean the segments: restricted to Oncoscan coverage, LOH not overlapping
# with copy loss segments, smooth&merge segments within 300kb and prune
# segments smaller than 300kb.
segs.clean <- trim_to_coverage(segments, oncoscan_na33.cov) %>%
    adjust_loh() %>%
    merge_segments() %>%
    prune_by_size()

# Then we need to compute the arm-level alteration for loss and LOH since many 
# scores discard arms that are globally altered.
arms.loss <- names(get_loss_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscan_na33.cov))
arms.loh <- names(get_loh_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscan_na33.cov))

# Get the number of LST
lst <- score_lst(segs.clean, oncoscan_na33.cov)

# Get the number of HR-LOH
hrloh <- score_loh(segs.clean, arms.loh, arms.loss, oncoscan_na33.cov)

# Get the genomic LOH score
gloh <- score_gloh(segs.clean, arms.loh, arms.loss, oncoscan_na33.cov)

# Get the number of nLST
wgd <- score_estwgd(segs.clean, oncoscan_na33.cov)  # Get the avg CN, including 21p
nlst <- score_nlst(segs.clean, wgd["WGD"], oncoscan_na33.cov)

print(c(LST=lst, `HR-LOH`=hrloh, gLOH=gloh, nLST=nlst))

## -----------------------------------------------------------------------------
# Load data
chas.fn <- system.file("extdata", "TDplus_gene_list_full_location.txt", 
                       package = "oncoscanR")
segments <- load_chas(chas.fn, oncoscan_na33.cov)

# Clean the segments: restricted to Oncoscan coverage, LOH not overlapping
# with copy loss segments, smooth&merge segments within 300kb and prune
# segments smaller than 300kb.
segs.clean <- trim_to_coverage(segments, oncoscan_na33.cov) %>%
    adjust_loh() %>%
    merge_segments() %>%
    prune_by_size()

td <- score_td(segs.clean)
print(td$TDplus)

## -----------------------------------------------------------------------------
segs.filename <- system.file('extdata', 'LST_gene_list_full_location.txt', 
                             package = 'oncoscanR')
dat <- workflow_oncoscan.chas(segs.filename)

message(paste('Arms with copy loss:', 
              paste(dat$armlevel$LOSS, collapse = ', ')))
message(paste('Arms with copy gains:', 
              paste(dat$armlevel$GAIN, collapse = ', ')))
message(paste('HRD score:', dat$scores$HRD))

## -----------------------------------------------------------------------------
library(jsonlite)
segs.filename <- system.file('extdata', 'ascat_example.txt', 
                             package = 'oncoscanR')
dat <- workflow_oncoscan.ascat(segs.filename)
toJSON(dat, auto_unbox=TRUE, pretty=TRUE)

## -----------------------------------------------------------------------------
sessionInfo()

# input.R Functions to handle loading/cleaning of segments and reference files.
# Author: Yann Christinat Date: 19.9.2019

#' Load a ChAS text export file.
#'
#' @details The ChAS file is expected to have the following column names:
#' 'CN State' (number or empty), 'Type' (expected value: 'Gain', 'Loss' or
#' 'LOH') and 'Full Location' (in the format 'chr:start-end').
#'
#' The segments are attributed to each chromosome arm and split if necessary.
#'
#' @param filename Path to the ChAS file.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm by the kit.
#'
#' @return A \code{GRanges} object containing the segments, their copy number
#' (field \code{cn}), their copy
#' number types (field \code{cntype}). \code{cntype} contains either 'Gain',
#' 'Loss' or 'LOH'.
#' If the file contains twice the same segment or does not respect the format
#' specifications, then an error is
#' raised. NB. The chromosome name is in the format '1' and not 'chr1' and will
#' be transformed if needed.
#'
#' @export
#'
#' @import readr
#' @import GenomicRanges
#' @import IRanges
#' @importFrom methods is
#'
#' @examples
#' segs.filename <- system.file('extdata', 'chas_example.txt',
#'   package = 'oncoscanR')
#' segs.chas_example <- load_chas(segs.filename, oncoscan_na33.cov)
load_chas <- function(filename, kit.coverage) {
    # Entry check on arguments
    stopifnot(is(kit.coverage, "GRanges"))

    # Reads in the ChAS file
    cols <- cols_only(`CN State` = col_number(), Type = col_character(),
                      `Full Location` = col_character())
    oncoscan_table <- read_tsv(filename, comment = "#", col_names = TRUE,
                               col_types = cols)

    if (dim(oncoscan_table)[2] != 3) {
        stop("Parsing ChAS file failed.")
    }
    if (dim(oncoscan_table)[1] == 0) {
        warning("No segments loaded!")
        return(GRanges())
    }

    # Process the data into GRanges segments
    segs <- process_chas(oncoscan_table, kit.coverage)

    # Test for duplicated entries
    errmsg <- paste("The file", filename, "contains duplicated entries.")
    for (arm in unique(seqnames(segs))) {
        arm_segs <- sort(segs[seqnames(segs) == arm])
        if (length(arm_segs) > 1) {
            for (i in 2:length(arm_segs)) {
                segA <- arm_segs[i - 1]
                segB <- arm_segs[i]
                if (IRanges::start(segA) == IRanges::start(segB) &
                    IRanges::end(segA) == IRanges::end(segB) &
                    segA$cn.type == segB$cn.type) {
                    if (segA$cn.type == cntypes$LOH) {
                        stop(errmsg)
                    } else if (segA$cn == segB$cn) {
                        stop(errmsg)
                    }
                }
            }
        }
    }

    if (length(segs) == 0) {
        warning("No segments loaded!")
    }

    return(segs)
}


#' Load am ASCAT text export file.
#'
#' @details The ASCAT file is expected to have the following column names:
#' 'chr' (chromosome number), 'startpos' (first position of CNV segment),
#' 'endpos' (last position of CNV segment), 'nMajor' (Number of copies of the
#' major allele) and 'nMinor' (Number of copies of the minor allele).
#'
#' The segments are attributed to each chromosome arm and split if necessary.
#'
#' @param filename Path to the ASCAT file.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm by the kit.
#'
#' @return A \code{GRanges} object containing the segments, their copy number
#' (field \code{cn}), their copy
#' number types (field \code{cntype}). \code{cntype} contains either 'Gain',
#' 'Loss' or 'LOH'.
#' If the file contains twice the same segment or does not respect the format
#' specifications, then an error is raised.
#' NB. If the chromosome name is in the format '1' and not 'chr1' and will
#' be transformed if needed.
#'
#' @export
#'
#' @import readr
#' @import GenomicRanges
#' @import IRanges
#' @importFrom methods is
#'
#' @examples
#' segs.filename <- system.file('extdata', 'ascat_example.txt',
#'   package = 'oncoscanR')
#' segs.ascat_example <- load_ascat(segs.filename, oncoscan_na33.cov)
load_ascat <- function(filename, kit.coverage) {
    ascat.segments <- read.table(filename, header = TRUE, sep = '\t')
    segs <- ascat.segments[, c('chr', 'startpos', 'endpos', 'nMajor', 'nMinor')]
    segs$cn <- segs$nMajor + segs$nMinor
    segs$full.loc <- paste0(segs$chr, ':', segs$startpos, '-', segs$endpos)
    cn.type <- sapply(segs$cn, function(cn) {
        # NOTE: To adapt for the sexual chromosomes if the sample can be male!
        if (cn < 2) {
            return("Loss")
        }
        if (cn > 2) {
            return("Gain")
        }
        return(NA)
    })
    segments_table <- data.frame(
        `Full Location` = segs$full.loc[!is.na(cn.type)],
        `CN State` = segs$cn[!is.na(cn.type)],
        Type = cn.type[!is.na(cn.type)],
        check.names = FALSE
    )

    # Add LOH segments (not necessary for nLST computation)
    segs.loh <- segs[(segs$nMajor == 0 | segs$nMinor == 0) & segs$cn > 1, ]
    if (dim(segs.loh)[1] > 0) {
        dt.loh <- data.frame(`Full Location` = segs.loh$full.loc,
                             `CN State` = '',
                             Type = rep('LOH', dim(segs.loh)[1]),
                             check.names = FALSE)
        segments_table <- rbind(segments_table, dt.loh)
    }


    segs <- GRanges()
    if(dim(segments_table)[1] > 0){
        # Process the data into GRanges segments
        segs <- process_chas(segments_table, kit.coverage)

        # Test for duplicated entries
        errmsg <- paste("The file", filename, "contains duplicated entries.")
        for (arm in unique(seqnames(segs))) {
            arm_segs <- sort(segs[seqnames(segs) == arm])
            if (length(arm_segs) > 1) {
                for (i in 2:length(arm_segs)) {
                    segA <- arm_segs[i - 1]
                    segB <- arm_segs[i]
                    if (IRanges::start(segA) == IRanges::start(segB) &
                        IRanges::end(segA) == IRanges::end(segB) &
                        segA$cn.type == segB$cn.type) {
                        if (segA$cn.type == cntypes$LOH) {
                            stop(errmsg)
                        } else if (segA$cn == segB$cn) {
                            stop(errmsg)
                        }
                    }
                }
            }
        }
    }

    if (length(segs) == 0) {
        warning("No segments loaded!")
    }

    segs$cn <- as.numeric(segs$cn)
    return(segs)
}


#' Process ChAS table.
#'
#' @details Used in the load_chas function.
#'
#' @param oncoscan_table A tibble with the following column names:
#' 'CN State' (number or empty), 'Type' (expected value: 'Gain', 'Loss' or
#' 'LOH') and 'Full Location' (in the format 'chr:start-end').
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm by the kit.
#'
#' @return A \code{GRanges} object containing the segments, their copy number
#' (field \code{cn}), their copy
#' number types (field \code{cntype}). \code{cntype} contains either 'Gain',
#' 'Loss' or 'LOH'.
#'
#' @noRd
process_chas <- function(oncoscan_table, kit.coverage){
    # Allocate the place for the GRanges segments. At most we will end up with
    # twice as much segments as present in the raw data.
    segments_list <- vector(mode = "list", length = 2 * dim(oncoscan_table)[1])

    # Parse throuh all lines of the data
    counter <- 0
    for (i in seq(dim(oncoscan_table)[1])) {
        counter <- counter + 1
        # Start: Extract chr no, start, end, copy number

        # Full location from oncoscan file (remove commas or spaces if present)
        loc_cord <- gsub("[,.' ]", "", oncoscan_table$`Full Location`[i])
        loc_cord_list <- strsplit(loc_cord, split = ":", fixed = TRUE)[[1]]

        # seg chr no based on oncoscan file
        seg_chr <- loc_cord_list[1]
        seg_chr <- gsub("chr", "", seg_chr)  #remove 'chr' if present

        # seg coordinates
        coord_list <- strsplit(loc_cord_list[2], split = "-", fixed = TRUE)[[1]]
        seg_start <- as.numeric(coord_list[1])
        seg_end <- as.numeric(coord_list[2])

        seg_cn <- oncoscan_table$`CN State`[i]
        seg_cntype <- oncoscan_table$Type[i]

        # Test if copy number type is correct
        if (!(seg_cntype %in% cntypes)) {
            msg <- paste("The column \"Type\" should contain only the",
                         "following values:", paste(cntypes, collapse = ','),
                         "whereas", seg_cntype, "was found!")
            stop(msg)
        }

        # Get the arms
        parm <- kit.coverage[seqnames(kit.coverage) == paste0(seg_chr, "p")]
        qarm <- kit.coverage[seqnames(kit.coverage) == paste0(seg_chr, "q")]

        kit.arms <- levels(seqnames(kit.coverage))
        if (length(parm) > 0 || length(qarm) > 0) {
            if (length(parm) == 0 || seg_start > end(parm)) {
                # Then the segment is only in the q arm
                sn <- factor(paste0(seg_chr, "q"), levels = kit.arms)
                rg <- IRanges(start = seg_start, end = seg_end)
                seg <- GRanges(seqnames=sn, ranges=rg,
                               cn=seg_cn, cn.type=seg_cntype)
                segments_list[[counter]] <- seg
            } else if (length(qarm) == 0 || seg_end < start(qarm)) {
                # Then the segment is only in the p arm
                sn <- factor(paste0(seg_chr, "p"), levels = kit.arms)
                rg <- IRanges(start = seg_start, end = seg_end)
                seg <- GRanges(seqnames=sn, ranges=rg,
                               cn=seg_cn, cn.type=seg_cntype)
                segments_list[[counter]] <- seg
            } else {
                # Create a segment for each arm
                sn <- factor(paste0(seg_chr, c("p", "q")), levels = kit.arms)
                rg <- IRanges(start = rep(seg_start, 2), end = rep(seg_end, 2))
                seg <- GRanges(seqnames = sn, ranges = rg)

                # Get the segments overlapping with the arms
                o <- IRanges::findOverlapPairs(c(parm, qarm), seg)
                new_segs <- pintersect(o)
                new_segs$cn <- rep(seg_cn, length(new_segs))
                new_segs$cn.type <- rep(seg_cntype, length(new_segs))
                new_segs$hit <- NULL

                segments_list[[counter]] <- new_segs
            }
        }
    }

    segs <- do.call("c", unlist(segments_list))
    return(segs)
}


#' Load the oncoscan coverage BED file into a GenomicRanges object.
#'
#' @details Expects the following columns from the BED file (no header):
#'   1. Name of the chromosomal arm (e.g. "1p")
#'   2. Start position of the arm
#'   3. End position of the arm
#'
#' @param filename Path to the coverage BED file.
#'
#' @return A \code{GRanges} object containing the regions covered on each
#' chromosome arm.
#'
#' @export
#'
#' @import GenomicRanges
#' @import IRanges
#' @importFrom utils read.table
#'
#' @examples
#' oncoscan_na33.cov <- get_oncoscan_coverage_from_bed(
#'        system.file('extdata', 'Oncoscan.na33.r2.cov.processed.bed',
#'        package = 'oncoscanR'))
get_oncoscan_coverage_from_bed <- function(filename) {
    # Read the annotation file
    dat <- read.table(filename, sep = '\t', header = FALSE)
    colnames(dat) <- c('Arm', 'Start', 'Stop')

    cov <- GRanges(seqnames = factor(dat$Arm),
                   ranges = IRanges(start = dat$Start, end = dat$Stop))
    return(cov)
}


#' Return all segments of type LOH, independently of the copy number.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#'
#' @return A \code{GRanges} object containing the selected segments, their copy
#' number and copy number types.
#' @export
#'
#' @examples
#' segs.loh <- get_loh_segments(segs.chas_example)
get_loh_segments <- function(segments) {
    is_cn_segment(segments, raise_error = TRUE)

    return(segments[segments$cn.type == cntypes$LOH])
}

#' Return all segments with loss of 1 or 2 copies.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#'
#' @return A \code{GRanges} object containing the selected segments, their copy
#' number and copy number types.
#' @export
#'
#' @examples
#' segs.loh <- get_loh_segments(segs.chas_example)
get_loss_segments <- function(segments) {
    is_cn_segment(segments, raise_error = TRUE)

    return(segments[segments$cn.type == cntypes$Loss])
}

#' Return all segments with heterozygous loss.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#'
#' @return A \code{GRanges} object containing the selected segments, their copy
#' number and copy number types.
#' @export
#'
#' @examples
#' segs.hetloss <- get_hetloss_segments(segs.chas_example)
get_hetloss_segments <- function(segments) {
    is_cn_segment(segments, raise_error = TRUE)

    return(segments[segments$cn.type == cntypes$Loss & segments$cn >= 1])
}

#' Return all segments with homozygous loss.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#'
#' @return A \code{GRanges} object containing the selected segments, their copy
#' number and copy number types.
#' @export
#'
#' @examples
#' get_homloss_segments <- get_homloss_segments(segs.chas_example)
get_homloss_segments <- function(segments) {
    is_cn_segment(segments, raise_error = TRUE)

    return(segments[segments$cn.type == cntypes$Loss & segments$cn < 1])
}

#' Return all segments with gain of copies.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#'
#' @return A \code{GRanges} object containing the selected segments, their copy
#' number and copy number types.
#' @export
#'
#' @examples
#' segs.gain <- get_gain_segments(segs.chas_example)
get_gain_segments <- function(segments) {
    is_cn_segment(segments, raise_error = TRUE)

    return(segments[segments$cn.type == cntypes$Gain])
}

#' Return all segments with an amplification (5 or more copies).
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#'
#' @return A \code{GRanges} object containing the selected segments, their copy
#' number and copy number types.
#' @export
#'
#' @examples
#' segs.amp <- get_amp_segments(segs.chas_example)
get_amp_segments <- function(segments) {
    is_cn_segment(segments, raise_error = TRUE)

    return(segments[segments$cn.type == cntypes$Gain & segments$cn >= 5])
}


#' Trim segments with respect to the kit's coverage.
#'
#' @details All segments that are not entirely contained within the kit coverage
#'  will be trimmed to the coverage's limits.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param kit.coverage A \code{GRanges} object containing the regions covered on
#'  each chromosome arm.
#'
#' @return A \code{GRanges} object containing the cleaned segments, their copy
#' number and copy number types.
#' @export
#'
#' @import GenomicRanges
#' @import IRanges
#'
#' @examples
#' segs.trimmed <- trim_to_coverage(segs.chas_example, oncoscan_na33.cov)
trim_to_coverage <- function(segments, kit.coverage) {
    is_cn_segment(segments, raise_error = TRUE)

    if (length(segments) == 0) {
        return(segments)
    }

    # Apply on each arm...
    segs.clean <- lapply(unique(seqnames(segments)), function(arm) {
        # Trim segments wrt coverage
        arm.cov <- kit.coverage[seqnames(kit.coverage) == arm]
        o <- IRanges::findOverlapPairs(segments[seqnames(segments) == arm],
                                       arm.cov)
        new_segs <- sort(pintersect(o))
        if (length(new_segs) == 0) {
            return(new_segs)
        }
        new_segs$hit <- NULL
        return(new_segs)
    })
    return(do.call("c", unlist(segs.clean)))
}


#' Merge segments with respect to the kit resolution and the copy number.
#'
#' @details If two segments are at a distance smaller than the resolution, then
#' the segments are merged if the
#' share the same \code{cn} value. Note that the function does not look at the
#' copy number type or subtype but
#' only at the actual copy number to decide whether segments can be merged.
#'
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param kit.resolution Number >0 indicating the minimum segment size
#' detectable by the technique (in kilobases).
#' Defaults to the Oncoscan assay resolution outside of cancer genes: 300Kb.
#'
#' @return A \code{GRanges} object containing the cleaned segments, their copy
#' number and copy number types.
#' @export
#'
#' @import GenomicRanges
#'
#' @examples
#' segs.merged <- merge_segments(segs.chas_example)
#' segs.merged_50k <- merge_segments(segs.chas_example, 50)
merge_segments <- function(segments, kit.resolution = 300) {
    is_cn_segment(segments, raise_error = TRUE)

    if (kit.resolution < 1 / 1000) {
        stop("Kit resolution has to be greater than zero.")
    }

    if (length(segments) == 0) {
        return(segments)
    }

    # Go through each arm...
    segs.merged <- lapply(unique(seqnames(segments)), function(arm) {
        # Go through each copy number
        armsegs <- segments[seqnames(segments) == arm]
        armsegs.merged <- lapply(unique(armsegs$cn), function(cn) {
            segs <- NULL
            if (is.na(cn)) {
                segs <- armsegs[is.na(armsegs$cn)]
            } else {
                segs <- armsegs[!is.na(armsegs$cn) & armsegs$cn == cn]
            }
            new_segs <-
                reduce(segs, min.gapwidth = kit.resolution * 1000)
            # Re-set the copy number, cn type and subtype
            new_segs$cn <- cn
            new_segs$cn.type <- segs[1]$cn.type
            if (!is.null(segs$cn.subtype)) {
                new_segs$cn.subtype <- segs[1]$cn.subtype
            }
            return(new_segs)
        })
        do.call("c", unlist(armsegs.merged))
    })
    return(do.call("c", unlist(segs.merged)))
}


#' Trim LOH segments with respect to loss segments.
#'
#' @details LOH segments completely contained within (or equal to)  a copy loss
#' segment are deleted.
#' LOH segments partially overlapping (on one end only) with a copy loss segment
#'  are trimmed to remove the overlap or split into several segments.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#'
#' @return A \code{GRanges} object containing the cleaned segments, their copy
#' number and copy number types.
#' @export
#'
#' @import GenomicRanges
#' @import IRanges
#' @import S4Vectors
#'
#' @examples
#' segs.adj <- adjust_loh(segs.chas_example)
adjust_loh <- function(segments) {
    is_cn_segment(segments, raise_error = TRUE)
    if (length(segments) == 0) { return(segments) }

    loh.segs <- segments[segments$cn.type == cntypes$LOH]
    if (length(loh.segs) == 0) {
        return(segments)  # Nothing to do if no LOH segments
    }

    # Apply on each arm
    loh.adj <- lapply(unique(seqnames(loh.segs)), function(arm) {
        armsegs.loh <- loh.segs[seqnames(loh.segs) == arm]

        if (length(armsegs.loh) == 0) { return(GRanges()) }
        armsegs.loss <- segments[segments$cn.type == cntypes$Loss &
                                  seqnames(segments) == arm]

        pos.all <- sort(unique(c(start(armsegs.loh), end(armsegs.loh)+1,
                                 start(armsegs.loss), end(armsegs.loss)+1)))
        dt <- data.frame(
            row.names = pos.all,
            loss = factor(rep('nd', length(pos.all)),
                          levels = c('nd', 'start', 'end')),
            loh = factor(rep('nd', length(pos.all)),
                         levels = c('nd', 'start', 'end'))
        )
        dt[as.character(start(armsegs.loh)), 'loh'] <- 'start'
        dt[as.character(end(armsegs.loh)+1), 'loh'] <- 'end'
        dt[as.character(start(armsegs.loss)), 'loss'] <- 'start'
        dt[as.character(end(armsegs.loss)+1), 'loss'] <- 'end'

        loh.toadd <- getLOHtoadd(dt)

        if (length(loh.toadd) == 0) {
            return(GRanges())
        }
        else {
            return(GRanges(seqnames = rep(arm, length(loh.toadd)),
                           ranges = loh.toadd
            ))
        }
    })

    new.loh <- do.call("c", unlist(loh.adj))
    if (length(new.loh) != 0) {
        new.loh$cn <- as.numeric(NA)
        new.loh$cn.type <- as.character(cntypes$LOH)
        if (!is.null(segments$cn.subtype)) {
            new.loh$cn.subtype <- as.character(cntypes$LOH)
        }
    }

    return(c(segments[segments$cn.type != cntypes$LOH], new.loh))
}


#' Given a list of segments, trims or split the LOH segments if they overlap
#' with a loss.
#'
#' @details Used in the adjust_loh function
#'
#' @param dt A data.frame with the position as row names and two columns: 'loh'
#' and 'loss' with values 'end' or 'start', indicating the starts and ends of
#' the segments.
#'
#' @return a list of IRanges segments
#'
#' @noRd
getLOHtoadd <- function(dt){
    in.loh <- FALSE
    in.loss <- FALSE
    lohseg.start <- NULL

    loh.toadd <- IRanges()
    for (i in seq(dim(dt)[1])) {
        # Update status whether we are in a loss
        in.loss <- ifelse(dt[i, 'loss'] == 'start', TRUE,
                          ifelse(dt[i, 'loss'] == 'end', FALSE, in.loss))

        # Update status whether we are in a LOH
        in.loh <- ifelse(dt[i, 'loh'] == 'start', TRUE,
                         ifelse(dt[i, 'loh'] == 'end', FALSE, in.loh))

        if (in.loh & !in.loss) {
            # Start of a LOH segment
            lohseg.start <- ifelse(dt[i, 'loss'] == 'end',
                                   as.numeric(rownames(dt)[i]),
                                   as.numeric(rownames(dt)[i]))
        }
        else if ((dt[i, 'loh'] == 'nd' && dt[i, 'loss'] == 'start' && in.loh) ||
                 (dt[i, 'loh'] == 'end' && dt[i, 'loss'] == 'start') ||
                 (dt[i, 'loh'] == 'end' && dt[i, 'loss'] == 'nd' && !in.loss)) {
            # End of a LOH segment
            lohseg.end <- ifelse(dt[i, 'loss'] == 'start',
                                 as.numeric(rownames(dt)[i]),
                                 as.numeric(rownames(dt)[i]))

            # Add segment
            loh.toadd <- append(loh.toadd,
                                IRanges(start = lohseg.start, end = lohseg.end-1))
            }
    }

    return(loh.toadd)
}

#' Remove segments smaller than the kit resolution.
#'
#' @param segments A \code{GRanges} object containing the segments, their copy
#' number and copy number types.
#' @param threshold Number indicating the minimum segment size to be kept (in
#' kilobases).
#' Defaults to the Oncoscan assay resolution outside of cancer genes: 300Kb.
#'
#' @return A \code{GRanges} object containing the cleaned segments, their copy
#' number and copy number types.
#' @export
#'
#' @import IRanges
#'
#' @examples
#' segs.300k <- prune_by_size(segs.chas_example)
#' segs.50k <- prune_by_size(segs.chas_example, 50)
prune_by_size <- function(segments, threshold = 300) {
    is_cn_segment(segments, raise_error = TRUE)

    if (threshold < 0) {
        stop("Threshold has to be greater than or equal to zero.")
    }
    if (length(segments) == 0) {
        return(segments)
    }

    return(segments[width(segments) >= threshold * 1000])
}

test_that("TD score works - only gains, border checks", {
  cov <- GRanges(seqnames = factor(c(paste0(1:4, 'p'), paste0(1:4, 'q')),
                                   levels = c(paste0(1:4, 'p'), paste0(1:4, 'q'))),
                 ranges = IRanges(start = rep(1, 8),
                                  end = rep(200*10^6, 8)))
  segs <- GRanges(seqnames = factor(c(rep('1p', 5), rep('2p', 4)), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c(1,   2,   3,  5, 20.000001, 50, 90.000001, 95, 100)*10^6 + 1,
                                   end =   c(1.1, 2.3, 4, 15, 30,        80, 91, 96.000001, 110.000001)*10^6),
                  cn = c(3, 4, 3, 4, 3, 4, 3, 4, 3),
                  cn.type = rep("Gain", 9))
  n <- score_td(segs)
  expected <- list(TDplus=3, TD=4)
  expect_equal(n, expected)
})

test_that("TD score works - CN checks", {
  cov <- GRanges(seqnames = factor(c(paste0(1:4, 'p'), paste0(1:4, 'q')),
                                   levels = c(paste0(1:4, 'p'), paste0(1:4, 'q'))),
                 ranges = IRanges(start = rep(1, 8),
                                  end = rep(200*10^6, 8)))
  segs <- GRanges(seqnames = factor(c(rep('1p', 7), rep('2p', 7)), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = rep(c(1,   10),7)*10^6 + 1,
                                   end =   rep(c(1.5, 15),7)*10^6),
                  cn = c(0,0,1,1,NA,NA,3,3,4,4,5,5,10,10),
                  cn.type = c("Loss", "Loss",
                              "Loss", "Loss",
                              "LOH", "LOH",
                              "Gain", "Gain",
                              "Gain", "Gain",
                              "Gain", "Gain",
                              "Gain", "Gain"))
  n <- score_td(segs)
  expected <- list(TDplus=2, TD=2)
  expect_equal(n, expected)
})

test_that("TD scores work - real case", {
  chas.fn <- system.file("extdata", "TDplus_gene_list_full_location.txt",
                         package = "oncoscanR")

  segments <- load_chas(chas.fn, oncoscanR::oncoscan_na33.cov)
  segs.clean <- trim_to_coverage(segments, oncoscanR::oncoscan_na33.cov) %>%
    prune_by_size()

  n <- score_td(segs.clean)
  expect_equal(n, list(TDplus=100, TD=22))  #Verified by hand in Excel and ChAS
})

test_that("TD scores work - empty segments", {
  segs <- GRanges(seqnames = factor(c(), levels = paste0(1:4, 'p')),
                  ranges = IRanges())
  n <- score_td(segs)
  expect_identical(unlist(n), c(TDplus=0,TD=0))
})

test_that("loading ChAS file works", {
  segs.filename <- system.file("extdata", "chas_example.txt",
                               package = "oncoscanR")
  segs <- load_chas(segs.filename, kit.coverage = oncoscan_na33.cov)

  split.segs <- GRanges(seqnames = c('7p','7q'),
                        ranges=IRanges(start=c(55219200, 61545024),
                                       end = c(58025423, 116328000)))
  split.segs$cn <- c(7, 7)
  split.segs$cn.type <- c('Gain', 'Gain')


  found <- 0
  for(i in seq_along(split.segs)){
    segA <- split.segs[i]
    for(j in seq_along(segs)){
      segB <- segs[j]
      if(same_segments(segA, segB)){
        found <- found+1
      }
    }
  }

  expect_equal(length(segs), 15)  
  # 14 segments in file but one is covering two arms
  expect_equal(found, 2, info = print(found))

})

test_that("loading ChAS file with separators in Full Location column works", {
    segs.filename <- "../testdata/chas_example-withSeparators.txt"
    segs <- load_chas(segs.filename, kit.coverage = oncoscan_na33.cov)
    
    split.segs <- GRanges(seqnames = c('7p','7q'),
                          ranges=IRanges(start=c(55219200, 61545024),
                                         end = c(58025423, 116328000)))
    split.segs$cn <- c(7, 7)
    split.segs$cn.type <- c('Gain', 'Gain')
    
    
    found <- 0
    for(i in seq_along(split.segs)){
        segA <- split.segs[i]
        for(j in seq_along(segs)){
            segB <- segs[j]
            if(same_segments(segA, segB)){
                found <- found+1
            }
        }
    }
    
    expect_equal(length(segs), 15)  
    # 14 segments in file but one is covering two arms
    expect_equal(found, 2, info = print(found))
    
})

test_that("loading ChAS file with duplicated rows fails", {
  segs.filename <- "../testdata/chas_example-withDuplicates.txt"
  expect_error(load_chas(segs.filename, kit.coverage = oncoscan_na33.cov),
              'contains duplicated entries.')
})


test_that("loading large ChAS file", {
  segs.filename <- system.file("extdata", "LST_gene_list_full_location.txt",
                               package = "oncoscanR")
  segs <- load_chas(segs.filename, kit.coverage = oncoscan_na33.cov)
  expect_true(length(segs)>100) 
  # Mostly testing that it does not fail nor return zero segments
})


test_that("Loading ChAS with missing 'Full Location' column fails",{
  segs.filename <- "../testdata/chas_example-noFullLocation.txt"
  expect_error(suppressWarnings(load_chas(segs.filename, 
                                          kit.coverage = oncoscan_na33.cov)),
               'Parsing ChAS file failed')
})

test_that("loading ChAS with 'chr' name scheme works", {
  segs.filename <- "../testdata/chas_example-withChr.txt"
  segs <- load_chas(segs.filename, kit.coverage = oncoscan_na33.cov)

  split.segs <- GRanges(seqnames = c('7p','7q'),
                        ranges=IRanges(start=c(55219200, 61545024),
                                       end = c(58025423, 116328000)))
  split.segs$cn <- c(7, 7)
  split.segs$cn.type <- c('Gain', 'Gain')

  found <- 0
  for(i in seq_along(split.segs)){
    segA <- split.segs[i]
    for(j in seq_along(segs)){
      segB <- segs[j]
      if(same_segments(segA, segB)){
        found <- found+1
      }
    }
  }

  expect_equal(c(length(segs), found), c(15, 2)) 
  # 14 segments in file but one is covering two arms
})

test_that("Loading ChAS with empty file works",{
  segs.filename <- "../testdata/chas_example-empty.txt"
  expect_warning(segs <- load_chas(segs.filename, 
                                   kit.coverage = oncoscan_na33.cov),
                 'No segments loaded')
  expect_equal(length(segs), 0)
})

test_that("Loading ASCAT file works", {
    segs.filename <- system.file("extdata", "ascat_example.txt",
                                 package = "oncoscanR")
    segs <- load_ascat(segs.filename, kit.coverage = oncoscan_na33.cov)
    
    expected.segs <- GRanges(seqnames = c('1q','5p'),
                          ranges=IRanges(start=c(144009053, 38139),
                                         end = c(249212878, 46193462)))
    expected.segs$cn <- c(3, 3)
    expected.segs$cn.type <- c('Gain', 'Gain')
    
    
    found <- 0
    for(i in seq_along(expected.segs)){
        segA <- expected.segs[i]
        for(j in seq_along(segs)){
            segB <- segs[j]
            if(same_segments(segA, segB)){
                found <- found+1
            }
        }
    }
    
    expect_equal(length(segs), 2)  # 2 segments in file 
    expect_equal(found, 2, info = print(found))
})

test_that("Loading ASCAT with empty file works",{
    segs.filename <- "../testdata/ascat_example-empty.txt"
    expect_warning(segs <- load_ascat(segs.filename, 
                                      kit.coverage = oncoscan_na33.cov),
                   'No segments loaded')
    expect_equal(length(segs), 0)
})

test_that("HR-LOH score works - simple cases", {
  cov <- GRanges(seqnames = factor(c(paste0(1:4, 'p'), paste0(1:4, 'q')),
                                   levels = c(paste0(1:4, 'p'), paste0(1:4, 'q'))),
                 ranges = IRanges(start = rep(1, 8),
                                  end = rep(200*10^6, 8)))
  segs <- GRanges(seqnames = factor(c(rep('1p', 6), rep('2p', 2)), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c( 1, 10, 40, 60-0.000001, 80, 100,  1, 50)*10^6 + 1,
                                   end =   c(10, 30, 55, 75,        95, 120, 20, 80)*10^6),
                  cn = c(NA, NA, NA, NA, 3, 1, NA, 15),
                  cn.type = c("LOH", "LOH", "LOH", "LOH",
                              "Gain", "Loss", "LOH", "Gain"))
  n <- score_loh(segs, c('2p', '2q'), c(), cov)
  expect_equal(n, 3)
})

test_that("HR-LOH score works - chrom with LOH and hetloss", {
  cov <- GRanges(seqnames = factor(c(paste0(1:4, 'p'), paste0(1:4, 'q')),
                                   levels = c(paste0(1:4, 'p'), paste0(1:4, 'q'))),
                 ranges = IRanges(start = rep(1, 8),
                                  end = rep(200*10^6, 8)))
  segs <- GRanges(seqnames = factor(c(rep('1p', 6), rep('2p', 2)), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c( 1, 10, 40, 60-0.000001, 80, 100,  1, 50)*10^6 + 1,
                                   end =   c(10, 30, 55, 75,        95, 120, 20, 80)*10^6),
                  cn = c(NA, NA, NA, NA, 3, 1, NA, 15),
                  cn.type = c("LOH", "LOH", "LOH", "LOH",
                              "Gain", "Loss", "LOH", "Gain"))
  n <- score_loh(segs, c('2p'), c('2q'), cov)
  expect_equal(n, 4)
})


test_that("HR-LOH score works - overlapping loh", {
  cov <- GRanges(seqnames = factor(c(paste0(1:4, 'p'), paste0(1:4, 'q')),
                                   levels = c(paste0(1:4, 'p'), paste0(1:4, 'q'))),
                 ranges = IRanges(start = rep(1, 8),
                                  end = rep(200*10^6, 8)))
  segs <- GRanges(seqnames = factor(c(rep('1p', 6), rep('2p', 2)), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c( 1, 10, 40, 60-0.000001, 80, 100,  1, 50)*10^6 + 1,
                                   end =   c(10, 20, 55, 75,        95, 120, 20, 80)*10^6),
                  cn = c(NA, NA, NA, NA, 3, 1, NA, 15),
                  cn.type = c("LOH", "LOH", "LOH", "LOH",
                              "Gain", "Loss", "LOH", "Gain"))
  n <- score_loh(segs, c(), c(), cov)
  expect_equal(n, 4)
})

test_that("LOH works - real case", {
  oncoscan.cov <- oncoscanR::oncoscan_na33.cov[seqnames(oncoscanR::oncoscan_na33.cov) != '21p']

  chas.fn <- system.file("extdata", "LST_gene_list_full_location.txt",
                         package = "oncoscanR")
  segments <- load_chas(chas.fn, oncoscan.cov)

  segs.clean <- trim_to_coverage(segments, oncoscan.cov) %>%
    adjust_loh() %>%
    prune_by_size()

  armlevel.loh <- get_loh_segments(segs.clean) %>%
    armlevel_alt(kit.coverage = oncoscan.cov)
  armlevel.hetloss <- get_hetloss_segments(segs.clean) %>%
    armlevel_alt(kit.coverage = oncoscan.cov)

  n <- score_loh(segs.clean, names(armlevel.loh), names(armlevel.hetloss), oncoscan.cov)
  expect_equal(n, 25) #Verified by hand in ChAS
})

test_that("LOH works - empty segments", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = c(1,1,101,101),
                                  end = c(100,100,200,200)))
  segs <- GRanges(seqnames = factor(c(), levels = paste0(1:4, 'p')),
                  ranges = IRanges())
  n <- score_loh(segs, c(), c(), cov)
  expect_equal(n, 0)
})

test_that("gLOH works - real case", {
  oncoscan.cov <- oncoscanR::oncoscan_na33.cov[seqnames(oncoscanR::oncoscan_na33.cov) != '21p']

  chas.fn <- system.file("extdata", "LST_gene_list_full_location.txt",
                         package = "oncoscanR")
  segments <- load_chas(chas.fn, oncoscan.cov)
  segs.clean <- trim_to_coverage(segments, oncoscan.cov) %>%
    adjust_loh() %>%
    prune_by_size()

  armlevel.loh <- get_loh_segments(segs.clean) %>%
    armlevel_alt(kit.coverage = oncoscan.cov)
  armlevel.hetloss <- get_hetloss_segments(segs.clean) %>%
    armlevel_alt(kit.coverage = oncoscan.cov)

  sel.segs <- segs.clean$cn.type=="LOH" | (segs.clean$cn.type=="Loss" & segs.clean$cn>=1)
  sel.arms <- !as.vector(seqnames(segs.clean)) %in% c(names(armlevel.loh), names(armlevel.hetloss))
  expected_p <- sum(width(segs.clean[sel.segs & sel.arms]))/sum(width(oncoscan.cov))

  p <- score_gloh(segs.clean, names(armlevel.loh), names(armlevel.hetloss), oncoscan.cov)
  expect_equal(p, expected_p)
})

test_that("Trimming to coverage works", {
  cov <- GRanges(seqnames = factor(c('1p', '1q', 'Xp'), levels = c('1p', '1q', 'Xp')),
                 ranges = IRanges(start = c(100, 1000, 100),
                                  end = c(200, 2000, 200)))
  segs <- GRanges(seqnames = factor(rep('1p',6), levels = c('1p', '1q', 'Xp')),
                  ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                   end = c(50, 130, 170, 250, 300, 250)),
                  cn = c(0,1,NA,3,5,10),
                  cn.type = c("Loss", "Loss", "LOH", "Gain",
                              "Gain", "Gain"))

  segs.clean <- trim_to_coverage(segs, cov)

  expected_segs <- GRanges(seqnames = factor(rep('1p',4), levels = c('1p', '1q', 'Xp')),
                           ranges = IRanges(start = c(100, 130, 170, 100),
                                            end = c(130, 170, 200, 200)),
                           cn = c(1,NA,3,10),
                           cn.type = c("Loss", "LOH", "Gain", "Gain"))

  expect_true(same_segmentsets(segs.clean, expected_segs))
})

test_that("Trimming with empty coverage works", {
  cov <- GRanges(seqnames = factor(c(), levels = c('1p', '1q', 'Xp')),
                 ranges = IRanges())
  segs <- GRanges(seqnames = factor(rep('1p',6), levels = c('1p', '1q', 'Xp')),
                  ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                   end = c(50, 130, 170, 250, 300, 250)),
                  cn = c(0,1,NA,3,5,10),
                  cn.type = c("Loss", "Loss", "LOH", "Gain",
                              "Gain", "Gain"))

  segs.clean <- trim_to_coverage(segs, cov)
  expected_segs <- GRanges()

  expect_true(same_segmentsets(segs.clean, expected_segs))
})



test_that("Merging works with different cn", {
  segs <- GRanges(seqnames = factor(rep('1p',7), levels = c('1p', '1q', 'Xp')),
                  ranges = IRanges(start = c (1, 11, 29, 51, 69, 71, 94),
                                   end =   c(10, 20, 40, 60, 80, 90, 110)),
                  cn = c(0,0,0,0,5,NA,NA),
                  cn.type = c("Loss", "Loss", "Loss", "Loss",
                              "Gain", "LOH", "LOH"))

  segs.clean <- merge_segments(segs, 10/1000)

  expected_segs <- GRanges(seqnames = factor(rep('1p',4), levels = c('1p', '1q', 'Xp')),
                           ranges = IRanges(start = c (1, 51, 69,  71),
                                            end =   c(40, 60, 80, 110)),
                           cn = c(0,0,5,NA),
                           cn.type = c("Loss", "Loss", "Gain", "LOH"))

  expect_true(same_segmentsets(segs.clean, expected_segs))
})

test_that("Merging works with overlaps", {
  segs <- GRanges(seqnames = factor(c(rep('1p',5), rep('2q',5)), levels = c('1p', '2q')),
                  ranges = IRanges(start = c( 1,  6, 51, 41, 71,  1,  6, 51, 41, 71),
                                   end =   c(10, 20, 60, 70, 80, 10, 20, 60, 70, 80)),
                  cn = c(0,0,0,0,0,3,NA,NA,1,1),
                  cn.type = c(rep("Loss", 5), "Gain", "LOH", "LOH", "Loss", "Loss"))

  segs.clean <- merge_segments(segs, 10/1000)

  expected_segs <- GRanges(seqnames = factor(c(rep('1p',2), rep('2q',4)), levels = c('1p', '2q')),
                           ranges = IRanges(start = c( 1, 41,  1,  6, 51, 41),
                                            end =   c(20, 80, 10, 20, 60, 80)),
                           cn = c(0,0,3,NA,NA,1),
                           cn.type = c("Loss", "Loss", "Gain",
                                       "LOH", "LOH", "Loss"))

  expect_true(same_segmentsets(segs.clean, expected_segs))
})

test_that("Merging works with large threshold", {
  segs <- GRanges(seqnames = factor(c(rep('1p',5), rep('2q',6)), levels = c('1p', '2q')),
                  ranges = IRanges(start = c( 1,  6, 51, 41, 71,  1,  6, 51, 41, 71, 91),
                                   end =   c(10, 20, 60, 70, 80, 10, 20, 60, 70, 80, 100)),
                  cn = c(0,0,0,0,0,3,NA,NA,1,1, 3),
                  cn.type = c(rep("Loss", 5),
                              "Gain", "LOH", "LOH", "Loss", "Loss", "Gain"))

  segs.clean <- merge_segments(segs, 1)

  expected_segs <- GRanges(seqnames = factor(c('1p', rep('2q',3)), levels = c('1p', '2q')),
                           ranges = IRanges(start = c( 1,   1,  6, 41),
                                            end =   c(80, 100, 60, 80)),
                           cn = c(0,3,NA,1),
                           cn.type = c("Loss", "Gain", "LOH", "Loss"))

  expect_true(same_segmentsets(segs.clean, expected_segs))
})

test_that("Merging return an error with threshold=0", {
  segs <- GRanges(seqnames = factor(rep('1p',7), levels = c('1p', '1q', 'Xp')),
                  ranges = IRanges(start = c (1, 11, 29, 51, 69, 71, 94),
                                   end =   c(10, 20, 40, 60, 80, 90, 110)),
                  cn = c(0,0,0,0,5,NA,NA),
                  cn.type = c("Loss", "Loss", "Loss", "Loss",
                              "Gain", "LOH", "LOH"))

  expect_error(merge_segments(segs, 0), 'greater than zero')
})

test_that("Merging works with min threshold (1)", {
  segs <- GRanges(seqnames = factor(c(rep('1p',5), rep('2q',5)), levels = c('1p', '2q')),
                  ranges = IRanges(start = c( 1,  6, 51, 41, 71,  1,  6, 22, 41, 71),
                                   end =   c(10, 20, 60, 70, 80, 10, 20, 60, 70, 80)),
                  cn = c(0,0,0,0,0,3,NA,NA,1,1),
                  cn.type = c(rep("Loss", 5),
                              "Gain", "LOH", "LOH", "Loss", "Loss"))

  segs.clean <- merge_segments(segs, 1/1000)

  expected_segs <- GRanges(seqnames = factor(c(rep('1p',2), rep('2q',4)), levels = c('1p', '2q')),
                           ranges = IRanges(start = c( 1, 41,  1,  6, 22, 41),
                                            end =   c(20, 80, 10, 20, 60, 80)),
                           cn = c(0,0,3,NA,NA,1),
                           cn.type = c("Loss", "Loss", "Gain",
                                       "LOH", "LOH", "Loss"))

  expect_true(same_segmentsets(segs.clean, expected_segs))
})

#' Tests the following scenario (symbol + indicate that the LOH region should be trimmed)
#' LOH  ++++----
#' Loss ----
#'
#' LOH  ----++++
#' Loss     ----
#'
#' LOH  ----++++----
#' Loss     ----
#'
#' LOH      ++++
#' Loss ------------
#'
#' LOH      ----
#' Loss ----
#'
#' LOH  ----
#' Loss     ----
#'
#' LOH  ++++
#' Loss --------
#'
#' LOH      ++++
#' Loss --------
#'
#' LOH  ++++    ++++
#' Loss ------------
#'
#' LOH  ++++----++++
#' Loss ----    ----
#'
test_that("LOH are trimmed with respect to Loss", {
  segs_loh <- GRanges(seqnames = factor(rep('1p',11), levels = c('1p', '1q', 'Xp')),
                  ranges = IRanges(start = c( 1, 51,  81, 131, 171, 191, 221, 261, 281, 301, 321),
                                   end =   c(20, 70, 110, 140, 180, 200, 230, 270, 290, 310, 350)),
                  cn = rep(NA, 11),
                  cn.type = rep("LOH", 11))
  segs_loss <- GRanges(seqnames = factor(rep('1p',11), levels = c('1p', '1q', 'Xp')),
                      ranges = IRanges(start = c( 1, 61,  91, 121, 161, 201, 221, 251, 281, 321, 341),
                                       end =   c(10, 70, 100, 150, 170, 210, 240, 270, 310, 330, 350)),
                      cn = c(rep(0, 3), rep(1, 3), rep(1.5, 5)),
                      cn.type = rep("Loss", 11))
  segs <- c(segs_loh, segs_loss)
  segs.clean <- adjust_loh(segs)

  expected_lohsegs <- GRanges(seqnames = factor(rep('1p',7), levels = c('1p', '1q', 'Xp')),
                           ranges = IRanges(start = c(11, 51, 81, 101, 171, 191, 331),
                                            end =   c(20, 60, 90, 110, 180, 200, 340)),
                           cn = rep(NA, 7),
                           cn.type = rep("LOH", 7))

  expect_true(same_segmentsets(segs.clean, c(expected_lohsegs, segs_loss)))
})


test_that("Pruning by size works", {
  segs <- GRanges(seqnames = factor(c('1p','2p','3p'), levels = c('1p','2p','3p')),
                  ranges = IRanges(start = c(1, 1, 1),
                                   end = c(49, 50, 51)),
                  cn = c(0,NA,5),
                  cn.type = c("Loss", "LOH", "Gain"))

  segs.clean <- prune_by_size(segs, 50/1000)

  expected_segs <- GRanges(seqnames = factor(c('2p','3p'), levels = c('1p','2p','3p')),
                  ranges = IRanges(start = c(1, 1),
                                   end = c(50, 51)),
                  cn = c(NA,5),
                  cn.type = c("LOH", "Gain"))

  expect_true(same_segmentsets(segs.clean, expected_segs))
})

test_that("Pruning by size works with zero threshold (no pruning)", {
  segs <- GRanges(seqnames = factor(c('1p','2p','3p'), levels = c('1p','2p','3p')),
                  ranges = IRanges(start = c(1, 1, 1),
                                   end = c(49, 50, 1)),
                  cn = c(0,NA,5),
                  cn.type = c("Loss", "LOH", "Gain"))

  segs.clean <- prune_by_size(segs, 0)

  expect_true(same_segmentsets(segs.clean, segs))
})

test_that("Cleaning with empty segments works", {
  cov <- GRanges(seqnames = factor(c('1p', '1q', 'Xp'), levels = c('1p', '1q', 'Xp')),
                 ranges = IRanges(start = c(100, 1000, 100),
                                  end = c(200, 2000, 200)))
  segs <- GRanges(seqnames = factor(c(), levels = c('1p', '1q', 'Xp')),
                  ranges = IRanges())
  segs.trimmed <- trim_to_coverage(segs, cov)
  segs.merged <- merge_segments(segs)
  segs.adj <- adjust_loh(segs)
  segs.pruned <- prune_by_size(segs)

  expect_true(same_segmentsets(segs, segs.trimmed))
  expect_true(same_segmentsets(segs, segs.merged))
  expect_true(same_segmentsets(segs, segs.adj))
  expect_true(same_segmentsets(segs, segs.pruned))
})

test_that("LST works - simple cases", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = rep(1, 4),
                                  end = rep(100*10^6, 4)))
  segs <- GRanges(seqnames = factor(c(rep('1p', 4), rep('2p', 2)), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c(30, 50, 70,  90, 25,  50)*10^6 + 1,
                                   end =   c(50, 70, 90, 100, 75, 100)*10^6),
                  cn = c(NA, 3, 1, 0, NA, 14),
                  cn.type = c("LOH", "Gain", "Loss", "Loss",
                              "LOH", "Gain"))
  n.1p <- score_lst(segs[seqnames(segs) == '1p'], cov)
  n.2p <- score_lst(segs[seqnames(segs) == '2p'], cov)
  n <- score_lst(segs, cov)
  expect_equal(c(n, n.1p, n.2p), c(7, 4, 3))
})

test_that("LST works - case 1", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = rep(1, 4),
                                  end = rep(100*10^6, 4)))
  segs <- GRanges(seqnames = factor(rep('1p', 2), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c(20,  30)*10^6 + 1,
                                   end =   c(22, 100)*10^6),
                  cn = c(3, 3),
                  cn.type = c("Gain", "Gain"))
  n <- score_lst(segs, cov)
  expect_equal(n, 1)
})

test_that("LST works - case 2", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = rep(1, 4),
                                  end = rep(100*10^6, 4)))
  segs <- GRanges(seqnames = factor(rep('1p', 3), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c(0,  20,  24)*10^6 + 1,
                                   end =   c(20, 22, 100)*10^6),
                  cn = c(1, 3, 3),
                  cn.type = c("Loss", "Gain", "Gain"))
  n <- score_lst(segs, cov)
  expect_equal(n, 0)
})

test_that("LST works - case 3", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = rep(1, 4),
                                  end = rep(100*10^6, 4)))
  segs <- GRanges(seqnames = factor(rep('1p', 2), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c(20,  31)*10^6 + 1,
                                   end =   c(29, 100)*10^6),
                  cn = c(3, 7),
                  cn.type = c("Gain", "Gain"))
  n <- score_lst(segs, cov)
  expect_equal(n, 2)
})

test_that("LST works - case 4", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = rep(1, 4),
                                  end = rep(100*10^6, 4)))
  segs <- GRanges(seqnames = factor(rep('1p', 2), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c(43,  50)*10^6 + 1,
                                   end =   c(57, 100)*10^6),
                  cn = c(NA, 7),
                  cn.type = c("LOH", "Gain"))
  n <- score_lst(segs, cov)
  expect_equal(n, 0)
})

test_that("LST works - case 5", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = rep(1, 4),
                                  end = rep(100*10^6, 4)))
  segs <- GRanges(seqnames = factor(rep('1p', 2), levels = paste0(1:4, 'p')),
                  ranges = IRanges(start = c(31, 61)*10^6 + 1,
                                   end =   c(80, 100)*10^6),
                  cn = c(3, 7),
                  cn.type = c("Gain", "Gain"))
  n <- score_lst(segs, cov)
  expect_equal(n, 3)
})

test_that("LST works - real case", {
  oncoscan.cov <- oncoscanR::oncoscan_na33.cov[seqnames(oncoscanR::oncoscan_na33.cov) != '21p']

  chas.fn <- system.file("extdata", "LST_gene_list_full_location.txt",
                         package = "oncoscanR")
  segments <- load_chas(chas.fn, oncoscan.cov)

  segs.clean <- trim_to_coverage(segments, oncoscan.cov) %>%
    adjust_loh() %>%
    merge_segments() %>%
    prune_by_size()

  n <- score_lst(segs.clean, oncoscan.cov)
  expect_equal(n, 26) # Verified by hand in ChAS (on the 4 first chromosomes)
})

test_that("LST works - empty segments", {
  cov <- GRanges(seqnames = factor(paste0(1:4, 'p'), levels = paste0(1:4, 'p')),
                 ranges = IRanges(start = c(1,1,101,101),
                                  end = c(100,100,200,200)))
  segs <- GRanges(seqnames = factor(c(), levels = paste0(1:4, 'p')),
                  ranges = IRanges())
  n <- score_lst(segs, cov)
  expect_equal(n, 0)
})

#!/usr/bin/Rscript
#
# Run the standard workflow for the detection of arm-level alterations and
# scores calculation and prints the output in JSON.
#
# Author: Yann Christinat
# Date: 9.8.2021

suppressPackageStartupMessages(library(magrittr))
suppressPackageStartupMessages(library(jsonlite))
suppressPackageStartupMessages(library(GenomicRanges))
suppressPackageStartupMessages(library(IRanges))
suppressPackageStartupMessages(library(oncoscanR))

#' Retrieve arguments from command line.
#' Expects that the first argument is the ChAS file.
args <- commandArgs(TRUE)
if(length(args) != 1){
  stop("The first argument has to be the ChAS file ")
}

chas.fn <- args[1]

dat <- workflow_oncoscan.chas(chas.fn)
print(toJSON(dat, auto_unbox=TRUE, pretty=TRUE))

# objects_doc.R Documentation of all R objects included in the package
# Author: Yann Christinat
# Date: 30.6.2022


#' GenomicRanges object of the chromosomal arms coverage for the oncoscan assay
#' (based on file extdata/Oncoscan.na33.r2.cov.processed.bed).
#'
#' @source \code{oncoscan_na33.cov <- get_oncoscan_coverage_from_bed(
#'     system.file('extdata', 'Oncoscan.na33.r2.cov.processed.bed',
#'                 package = 'oncoscanR'))}
#'
#' @format A \code{GRanges} object containing the region covered on each
#' chromosome arm.
"oncoscan_na33.cov"


#' Expected segments from loading the ChAS file 'chas_example.txt'.
#'
#' @source
#' segs.filename <- system.file('extdata', 'chas_example.txt',
#' package = 'oncoscanR')
#' mykit.cov <- get_oncoscan_coverage_from_probes()
#' segs.chas_example <- load_chas(segs.filename, kit.coverage = mykit.cov)
#'
#' @format A \code{GRanges} object containing the segments, their copy number
#' (field \code{cn}) and their copy number types (field \code{cn.type}).
"segs.chas_example"

#' Accepted types of CN for the segments
#'   - 'Gain': 1-2 extra copies
#'   - 'Weak amplification': 3-7 extra copies
#'   - 'Strong amplification': 8 or more extra copies
#'   - 'Heterozygote loss': Loss of one copy out of two
#'   - 'Homozygote loss': Loss of all copies
#'   - 'LOH': copy-neutral loss of one parental allele
#'
#' @source
#' cntypes <- list(LOH='LOH', Gain='Gain', Loss='Loss')
#'
#' @format A list of strings containing all CN types
"cntypes"

# workflow_oncoscan.R Functions to run the complete workflow from input files
# to scores and arm-level alterations.  Author: Yann Christinat Date:
# 23.06.2020

#' Run the standard workflow for Oncoscan ChAS files.
#'
#' @details Identifies the globally altered arms (\>=90\% of arm altered),
#' computes the HRD and TD+ scores. The amplification is defined as a CN subtype
#'  \code{cntype.weakamp} or \code{cntype.strongamp}. An arm is gained if of CN
#'  type \code{cntype.gain} unless the arm is amplified.
#'
#' @param chas.fn Path to the text-export ChAS file
#'
#' @return A list of lists with the following elements:
#' \code{armlevel = list(AMP= list of arms, GAIN= list of arms, LOSS= list of
#' arms, LOH= list of arms),
#' scores = list(LST= number, LOH= number, TDplus= number, TD= number),
#' file = path of the ChAS file as given by the parameter)}
#'
#' @export
#'
#' @importFrom magrittr %>%
#'
#' @examples
#' segs.filename <- system.file('extdata', 'chas_example.txt',
#' package = 'oncoscanR')
#' workflow_oncoscan.chas(segs.filename)
workflow_oncoscan.chas <- function(chas.fn) {
    # Load the ChAS file and assign subtypes.
    segments <- load_chas(chas.fn, oncoscanR::oncoscan_na33.cov)

    # Clean the segments: resctricted to Oncoscan coverage, LOH not overlapping
    # with copy loss segments, smooth&merge segments within 300kb and prune
    # segments smaller than 300kb.
    segs.clean <- trim_to_coverage(segments, oncoscanR::oncoscan_na33.cov) %>%
        adjust_loh() %>%
        merge_segments() %>%
        prune_by_size()

    # Split segments by type: Loss, LOH, gain or amplification and get the
    # arm-level alterations.  Note that the segments with copy gains include
    # all amplified segments.
    armlevel.loss <- get_loss_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscanR::oncoscan_na33.cov)
    armlevel.loh <- get_loh_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscanR::oncoscan_na33.cov)
    armlevel.gain <- get_gain_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscanR::oncoscan_na33.cov)
    armlevel.amp <- get_amp_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscanR::oncoscan_na33.cov)

    # Remove amplified segments from armlevel.gain
    armlevel.gain <-
        armlevel.gain[!(names(armlevel.gain) %in% names(armlevel.amp))]

    # Get the number of nLST and TDplus
    wgd <- score_estwgd(segs.clean, oncoscanR::oncoscan_na33.cov)
    hrd <- score_nlst(segs.clean, wgd["WGD"], oncoscanR::oncoscan_na33.cov)

    n.td <- score_td(segs.clean)

    mbalt <- score_mbalt(segs.clean, oncoscanR::oncoscan_na33.cov, loh.rm=TRUE)

    hrd.label <- hrd["HRD"]
    if (mbalt['sample'] / mbalt['kit'] < 0.01)
        hrd.label <- paste(hrd["HRD"], "(no tumor?)")

    # Get the alterations into a single list and print it in a JSON format.
    armlevel_alt.list <-
        list(
            AMP = sort(names(armlevel.amp)),
            LOSS = sort(names(armlevel.loss)),
            LOH = sort(names(armlevel.loh)),
            GAIN = sort(names(armlevel.gain))
        )
    scores.list <-
        list(
            HRD = paste0(hrd.label, ", nLST=", hrd["nLST"]),
            TDplus = n.td$TDplus,
            avgCN = substr(as.character(wgd["avgCN"]), 1, 4)
        )

    return(list(
        armlevel = armlevel_alt.list,
        scores = scores.list,
        file = basename(chas.fn)
    ))
}

#' Run the standard workflow for ASCAT files (from oncoscan data).
#'
#' @details Identifies the globally altered arms (\>=90\% of arm altered),
#' computes the HRD and TD+ scores. The amplification is defined as a CN>=5. 
#' An arm is gained if of CN type \code{cntype.gain} unless the arm is 
#' amplified.
#'
#' @param ascat.fn Path to the text-export ASCAT file
#'
#' @return A list of lists with the following elements:
#' \code{armlevel = list(AMP= list of arms, GAIN= list of arms, LOSS= list of
#' arms, LOH= list of arms),
#' scores = list(LST= number, LOH= number, TDplus= number, TD= number),
#' file = path of the ChAS file as given by the parameter)}
#'
#' @export
#'
#' @importFrom magrittr %>%
#'
#' @examples
#' segs.filename <- system.file('extdata', 'ascat_example.txt',
#' package = 'oncoscanR')
#' workflow_oncoscan.ascat(segs.filename)
workflow_oncoscan.ascat <- function(ascat.fn) {
    # Load the ChAS file and assign subtypes.
    segments <- load_ascat(ascat.fn, oncoscanR::oncoscan_na33.cov)
    
    # Clean the segments: resctricted to Oncoscan coverage, LOH not overlapping
    # with copy loss segments, smooth&merge segments within 300kb and prune
    # segments smaller than 300kb.
    segs.clean <- trim_to_coverage(segments, oncoscanR::oncoscan_na33.cov) %>%
        adjust_loh() %>%
        merge_segments() %>%
        prune_by_size()
    
    # Split segments by type: Loss, LOH, gain or amplification and get the
    # arm-level alterations.  Note that the segments with copy gains include
    # all amplified segments.
    armlevel.loss <- get_loss_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscanR::oncoscan_na33.cov)
    armlevel.loh <- get_loh_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscanR::oncoscan_na33.cov)
    armlevel.gain <- get_gain_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscanR::oncoscan_na33.cov)
    armlevel.amp <- get_amp_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscanR::oncoscan_na33.cov)
    
    # Remove amplified segments from armlevel.gain
    armlevel.gain <-
        armlevel.gain[!(names(armlevel.gain) %in% names(armlevel.amp))]
    
    # Get the number of nLST and TDplus
    wgd <- score_estwgd(segs.clean, oncoscanR::oncoscan_na33.cov)
    hrd <- score_nlst(segs.clean, wgd["WGD"], oncoscanR::oncoscan_na33.cov)
    
    n.td <- score_td(segs.clean)
    
    mbalt <- score_mbalt(segs.clean, oncoscanR::oncoscan_na33.cov, loh.rm=TRUE)
    
    hrd.label <- hrd["HRD"]
    if (mbalt['sample'] / mbalt['kit'] < 0.01)
        hrd.label <- paste(hrd["HRD"], "(no tumor?)")
    
    # Get the alterations into a single list and print it in a JSON format.
    armlevel_alt.list <-
        list(
            AMP = sort(names(armlevel.amp)),
            LOSS = sort(names(armlevel.loss)),
            LOH = sort(names(armlevel.loh)),
            GAIN = sort(names(armlevel.gain))
        )
    scores.list <-
        list(
            HRD = paste0(hrd.label, ", nLST=", hrd["nLST"]),
            TDplus = n.td$TDplus,
            avgCN = substr(as.character(wgd["avgCN"]), 1, 4)
        )
    
    return(list(
        armlevel = armlevel_alt.list,
        scores = scores.list,
        file = basename(ascat.fn)
    ))
}

test_that("Oncoscan workflow works - LST", {
  #skip_on_cran()

  chas.fn <- system.file("extdata", "LST_gene_list_full_location.txt",
                         package = "oncoscanR")
  armlevel.fn <- "../testdata/LST_gene_list_full_location.armlevel_scna.csv"

  # Run standard workflow from package
  dat <- workflow_oncoscan.chas(chas.fn)

  tests <- armlevel.test(dat[['armlevel']], armlevel.fn)
  expect_true(sum(tests)==4)

  scores <- dat[['scores']]
  expect_equal(as.character(scores['HRD']), "Positive, nLST=22.5")
  expect_equal(as.character(scores['avgCN']), '2.64')
  expect_equal(as.character(scores['TDplus']), '128')
})

test_that("Oncoscan workflow works - triploide", {
  #skip_on_cran()

  chas.fn <- system.file("extdata", "triploide_gene_list_full_location.txt",
                         package = "oncoscanR")
  armlevel.fn <- "../testdata/triploide_gene_list_full_location.armlevel_scna.csv"

  # Run standard workflow from package
  dat <- workflow_oncoscan.chas(chas.fn)

  tests <- armlevel.test(dat[['armlevel']], armlevel.fn)
  expect_true(sum(tests)==4)

  scores <- dat[['scores']]
  expect_equal(as.character(scores['HRD']), "Negative, nLST=0")
  expect_equal(as.character(scores['avgCN']), '3.24')
  expect_equal(as.character(scores['TDplus']), '18')
})

test_that("Oncoscan workflow works - TDplus", {
  #skip_on_cran()

  chas.fn <- system.file("extdata", "TDplus_gene_list_full_location.txt",
                         package = "oncoscanR")
  armlevel.fn <- "../testdata/TDplus_gene_list_full_location.armlevel_scna.csv"

  # Run standard workflow from package
  dat <- workflow_oncoscan.chas(chas.fn)

  tests <- armlevel.test(dat[['armlevel']], armlevel.fn)
  expect_true(sum(tests)==4)

  scores <- dat[['scores']]
  expect_equal(as.character(scores['HRD']), "Negative, nLST=10.5")
  expect_equal(as.character(scores['avgCN']), '3.21')
  expect_equal(as.character(scores['TDplus']), '93')
})

test_that("Oncoscan workflow returns the correct structure", {
  segs.filename <- system.file("extdata", "chas_example.txt", package = "oncoscanR")
  dat <- workflow_oncoscan.chas(segs.filename)

  armlevel.nametest <- identical(sort(names(dat[['armlevel']])), sort(c('AMP', 'GAIN', 'LOH', 'LOSS')))
  scores.nametest <- identical(sort(names(dat[['scores']])), sort(c("avgCN", "HRD", "TDplus")))
  file.test <- dat[['file']] == basename(segs.filename)
  #print(c(armlevel.nametest, scores.nametest, file.test))

  expect_true(sum(c(armlevel.nametest, scores.nametest, file.test))==3)
})

test_that("Oncoscan workflow works with an empty file", {
  segs.filename <- "../testdata/chas_example-empty.txt"
  armlevel.fn <- "../testdata/chas_example-empty.armlevel_scna.csv"

  expect_warning(dat <- workflow_oncoscan.chas(segs.filename))

  # Test arm-level detection
  tests <- armlevel.test(dat[['armlevel']], armlevel.fn)
  expect_true(sum(tests)==4)

  # Test scores
  scores <- unlist(dat[['scores']])
  expect_equal(as.character(scores['HRD']), "Negative (no tumor?), nLST=0")
  expect_equal(as.character(scores['avgCN']), '2')
  expect_equal(as.character(scores['TDplus']), '0')
})

test_that("Oncoscan ASCAT workflow works", {
    #skip_on_cran()
    
    ascat.fn <- system.file("extdata", "ascat_example.txt",
                           package = "oncoscanR")
    armlevel.fn <- "../testdata/ascat_example.armlevel_scna.csv"
    
    # Run standard workflow from package
    dat <- workflow_oncoscan.ascat(ascat.fn)
    
    tests <- armlevel.test(dat[['armlevel']], armlevel.fn)
    expect_true(sum(tests)==4)
    
    scores <- dat[['scores']]
    expect_equal(as.character(scores['HRD']), "Negative, nLST=0")
    expect_equal(as.character(scores['avgCN']), '2.05')
    expect_equal(as.character(scores['TDplus']), '0')
})

test_that("ASCAT workflow works with an empty file", {
    segs.filename <- "../testdata/ascat_example-empty.txt"
    armlevel.fn <- "../testdata/chas_example-empty.armlevel_scna.csv"
    
    expect_warning(dat <- workflow_oncoscan.ascat(segs.filename))
    
    # Test arm-level detection
    tests <- armlevel.test(dat[['armlevel']], armlevel.fn)
    expect_true(sum(tests)==4)
    
    # Test scores
    scores <- unlist(dat[['scores']])
    expect_equal(as.character(scores['HRD']), "Negative (no tumor?), nLST=0")
    expect_equal(as.character(scores['avgCN']), '2')
    expect_equal(as.character(scores['TDplus']), '0')
})

#' Compare two CNV segments and test if equal
#'
#' @param x A \code{GRanges} object one segment, with optional fields \code{cn},
#'  \code{cn.type} and \code{cn.subtype}
#' @param y A \code{GRanges} object one segment, with optional fields \code{cn},
#'  \code{cn.type} and \code{cn.subtype}
#'
#' @return A boolean. TRUE if the segments' chromosome/arm, start/end and CN
#' tags match.
same_segments <- function(x, y) {
    # Test the class of the parameters
    if (!is(x, "GRanges") || !is(y, "GRanges")) {
        stop("Function 'compare_segments' can only handle GRanges objects.")
    }

    # Ensures that the GRanges contain only 1 segment each
    if (length(x) != length(y) || length(x) != 1) {
        msg <- paste("Function 'compare_segments' can only handle GRanges with only one segment. Found:",
                     length(x), length(y))
        stop(msg)
    }

    # Replace any NA by NULL in cn, cntype
    if (!is.null(x$cn) && is.na(x$cn)) {
        x$cn <- NULL
    }
    if (!is.null(x$cn.type) && is.na(x$cn.type)) {
        x$cn.type <- NULL
    }

    if (!is.null(y$cn) && is.na(y$cn)) {
        y$cn <- NULL
    }
    if (!is.null(y$cn.type) && is.na(y$cn.type)) {
        y$cn.type <- NULL
    }


    # Compare segment position and arm
    if (as.character(seqnames(x)) != as.character(seqnames(y)) ||
        start(x) != start(y) || end(x) != end(y)) {
        return(FALSE)
    } else {
        # Same segment: Test cn (if set)
        if ((is.null(x$cn) && !is.null(y$cn)) ||
            (!is.null(x$cn) && is.null(y$cn))) {
            # Only one cn is null
            return(FALSE)
        } else if (!is.null(x$cn) && !is.null(y$cn)) {
            # cn is set on both segments
            if (x$cn != y$cn) {
                return(FALSE)
            }
        }

        # Test cn.type (if set)
        if ((is.null(x$cn.type) && !is.null(y$cn.type)) ||
            (!is.null(x$cn.type) && is.null(y$cn.type))) {
            # Only one cn.type is null
            return(FALSE)
        } else if (!is.null(x$cn.type) & !is.null(y$cn.type)) {
            # cn is set on both segments
            if (x$cn.type != y$cn.type) {
                return(FALSE)
            }
        }
    }
    return(TRUE)
}


#' Small function to test if two sets have the same CN segments
#'
#' @param grA A \code{GRanges} object with fields \code{cn} and \code{cn.type}
#' @param grB A \code{GRanges} object with fields \code{cn} and \code{cn.type}
#'
#' @return TRUE if the two sets contain exactly the same CN segments
#'
#' @noRd
same_segmentsets <- function(grA, grB) {
    if (length(grA) != length(grB)) {
        return(FALSE)
    }

    found <- 0
    for (i in seq_along(grA)) {
        segA <- grA[i]
        for (j in seq_along(grB)) {
            segB <- grB[j]
            if (same_segments(segA, segB)) {
                found <- found + 1
            }
        }
    }
    return(found == length(grA))
}


#' Loads the results of the armlevel_alt function and compare it to the results
#' from a text file
armlevel.test <- function(dat.armlevel, armlevel.fn){
    # Get ground truth
    dat.true <- read.csv(armlevel.fn, header = TRUE, row.names = 1,
                         stringsAsFactors = FALSE, colClasses = 'character')

    #Check AMP, GAIN, LOSS and LOH
    amp.test <- identical(sort(dat.armlevel[['AMP']]),
                          sort(unlist(strsplit(dat.true['AMPL','Arms'], ','))))
    gain.test <- identical(sort(dat.armlevel[['GAIN']]),
                           sort(unlist(strsplit(dat.true['GAIN','Arms'], ','))))
    loh.test <- identical(sort(dat.armlevel[['LOH']]),
                          sort(unlist(strsplit(dat.true['LOH','Arms'], ','))))
    loss.test <- identical(sort(dat.armlevel[['LOSS']]),
                           sort(unlist(strsplit(dat.true['LOSS','Arms'], ','))))
    return(c(amp.test, gain.test, loh.test, loss.test))
}

test_that("Compare segments works - same segment, no cn info", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10))
  s2 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10))
  expect_true(same_segments(s1,s2))
})

test_that("Compare segments works - diff arm, no cn info", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10))
  s2 <- GRanges(seqnames = '1q', ranges = IRanges(start = 1, end = 10))
  expect_false(same_segments(s1,s2))
})

test_that("Compare segments works - diff end, no cn info", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10))
  s2 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 11))
  expect_false(same_segments(s1,s2))
})

test_that("Compare segments works - diff start, no cn info", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10))
  s2 <- GRanges(seqnames = '1p', ranges = IRanges(start = 2, end = 10))
  expect_false(same_segments(s1,s2))
})

test_that("Compare segments works - same segment, same cn", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 5, cn.type = "Gain")
  s2 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 5, cn.type = "Gain")
  expect_true(same_segments(s1,s2))
})

test_that("Compare segments works - same segment, diff cn", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 5, cn.type = "Gain")
  s2 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 6, cn.type = "Gain")
  expect_false(same_segments(s1,s2))
})

test_that("Compare segments works - same segment, NA cn", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 5, cn.type = "Gain")
  s2 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = NA, cn.type = "Gain")
  expect_false(same_segments(s1,s2))
})

test_that("Compare segments works - same segment, diff cn.type", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 5, cn.type = "Gain")
  s2 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 5, cn.type = "Amplification")
  expect_false(same_segments(s1,s2))
})

test_that("Compare segments works - same segment, null cn", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 5, cn.type = "Gain")
  s2 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn.type = "Gain")
  expect_false(same_segments(s1,s2))
})

test_that("Compare segments works - same segment, null cn.type", {
  s1 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 5, cn.type = "Gain")
  s2 <- GRanges(seqnames = '1p', ranges = IRanges(start = 1, end = 10),
                cn = 5)
  expect_false(same_segments(s1,s2))
})

test_that("Same set works - same segments/cn/cntype", {
  segs <- GRanges(seqnames = factor(c(rep('1p',3), rep('1q',3)),
                                    levels = c('1p', '1q', 'Xp')),
                  ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                   end = c(50, 130, 170, 250, 300, 250)),
                  cn = c(0,1,NA,3,5,10),
                  cn.type = c("Loss", "Loss", "LOH", "Gain",
                              "Gain", "Gain"))
  expect_true(same_segmentsets(segs, segs))
})

test_that("Same set works - different arm", {
  segs.a <- GRanges(seqnames = factor(c(rep('1p',3), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                     end = c(50, 130, 170, 250, 300, 250)),
                    cn = c(0,1,NA,3,5,10),
                    cn.type = c("Loss", "Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  segs.b <- GRanges(seqnames = factor(c(rep('1q',3), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                     end = c(50, 130, 170, 250, 300, 250)),
                    cn = c(0,1,NA,3,5,10),
                    cn.type = c("Loss", "Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  expect_false(same_segmentsets(segs.a, segs.b))
})

test_that("Same set works - different position", {
  segs.a <- GRanges(seqnames = factor(c(rep('1p',3), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                     end = c(50, 130, 170, 250, 300, 250)),
                    cn = c(0,1,NA,3,5,10),
                    cn.type = c("Loss", "Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  segs.b <- GRanges(seqnames = factor(c(rep('1p',3), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                     end = c(51, 130, 170, 250, 300, 250)),
                    cn = c(0,1,NA,3,5,10),
                    cn.type = c("Loss", "Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  expect_false(same_segmentsets(segs.a, segs.b))
})

test_that("Same set works - one segment missing", {
  segs.a <- GRanges(seqnames = factor(c(rep('1p',3), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                     end = c(50, 130, 170, 250, 300, 250)),
                    cn = c(0,1,NA,3,5,10),
                    cn.type = c("Loss", "Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  segs.b <- GRanges(seqnames = factor(c(rep('1p',2), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(50, 130, 170, 250, 50),
                                     end = c(130, 170, 250, 300, 250)),
                    cn = c(1,NA,3,5,10),
                    cn.type = c("Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  expect_false(same_segmentsets(segs.a, segs.b))
})

test_that("Same set works - different copy number", {
  segs.a <- GRanges(seqnames = factor(c(rep('1p',3), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                     end = c(50, 130, 170, 250, 300, 250)),
                    cn = c(0,1,NA,3,5,10),
                    cn.type = c("Loss", "Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  segs.b <- GRanges(seqnames = factor(c(rep('1p',3), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                     end = c(50, 130, 170, 250, 300, 250)),
                    cn = c(NA,1,NA,3,5,10),
                    cn.type = c("LOH", "Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  expect_false(same_segmentsets(segs.a, segs.b))
})

test_that("Same set works - different cntype", {
  segs.a <- GRanges(seqnames = factor(c(rep('1p',3), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                     end = c(50, 130, 170, 250, 300, 250)),
                    cn = c(0,1,NA,3,5,10),
                    cn.type = c("Loss", "Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  segs.b <- GRanges(seqnames = factor(c(rep('1p',3), rep('1q',3)),
                                      levels = c('1p', '1q', 'Xp')),
                    ranges = IRanges(start = c(1, 50, 130, 170, 250, 50),
                                     end = c(50, 130, 170, 250, 300, 250)),
                    cn = c(0,1,NA,3,5,10),
                    cn.type = c("LOH", "Loss", "LOH", "Gain",
                                "Gain", "Gain"))
  expect_false(same_segmentsets(segs.a, segs.b))
})

---
title: "Secondary analyses of CNV data (HRD and more) with oncoscanR"
author: "Yann Christinat"
date: "`r format(Sys.time(), '%B %d, %Y')`"
output:
    BiocStyle::html_document:
        number_sections: yes
        toc: true
vignette: >
    %\VignetteIndexEntry{oncoscanR vignette}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)
```

# OncoscanR package description
OncoscanR is an R package to handle Copy Number Variation analyses originating 
from the Oncoscan assay (Affymetrix). It allows computation of different 
homologous recombination deficiency (HRD) scores and the tandem duplication plus
score (TDplus) to identify CDK12-mutated tumors [Popova et al., 
Cancer Res 2016]. 
The package also allows for identification of arm-level alterations (e.g. gain 
of chromosome arm 1p).

## Inclusion in Bioconductor
The package allows secondary analysis of biological microarray data. It relies 
on existing bioconductor packages to enable rigourous and reproducible analyses.
The package could, for example, be used to compute an HRD score using CNV data 
from the `conumee` package. 

**IMPORTANT**: The package expects as input the text exported file from ChAS 
(Chromosome Analysis Suite; the Affymetrix software to identify CNV segments 
from the Oncoscan Assay). The package assumes that all segments given in the 
file are correct and true. The ChAS text file has to contain the columns `Type`,
`CN State` and `Full Location` (to setup in ChAS). Any text file that complies 
with this structure should work equally well.

Starting with version 1.3.0, ASCAT output files can also be used as input.

Note that the Oncoscan does not cover the p arms of chromosome 13, 14, 15 and 
22. 
The coverage on the p arm of chromosome 21 is only partial and is not included 
in the standard Oncoscan workflow (function `workflow_oncoscan.chas` or script 
`bin/oncoscan-workflow.R`).

# Getting started
## Installation

OncoscanR is available through Bioconductor. In R, enter the commands:

```{r eval=FALSE}
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")

BiocManager::install("oncoscanR")
```

You can also install the development version via GitHub.:
```{r eval=FALSE}
# install.packages('devtools')
install_github('yannchristinat/oncoscanR')
```

Note that the installation of the development version requires the prior 
installation of the packages `GenomicRanges` (bioconductor), `magrittr`, 
`jsonlite` and `readr`.

```{r eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("GenomicRanges")

install.packages(c("magrittr", "jsonlite", "readr"))
```

## Testing the installation
Open R and type the following commands:
```{r}
library(oncoscanR)
segs.filename <- system.file("extdata", "chas_example.txt", 
                             package = "oncoscanR")
res <- workflow_oncoscan.chas(segs.filename)
print(res)
```

If everything is setup fine, `res` should contain a list with no arm-level 
alterations and a negative HRD score (nLST=1).


# Use cases
## Loading a ChAS export file and do a bit of cleaning
ChAS exports files contain only basic information about the copy number (gain, 
loss or LOH), plus the segment may overlap the centromere. 
When the file is loaded by OncoscanR (`load_chas` function), all segments are 
assigend to a chromosomal arms and split if necessary.
The LOH segments are given by ChAS independently of the copy number variation 
segments. Therefore one may have a LOH segment overlapping with a copy loss. As 
this information is redundant (a copy loss will always have a LOH), we need to 
trim and split these LOH with the `adjust_loh` function.

```{r}
library(magrittr)

# Load the ChAS file
chas.fn <- system.file("extdata", "chas_example.txt", package = "oncoscanR")
segments <- load_chas(chas.fn, oncoscan_na33.cov)

# Clean the segments: restricted to Oncoscan coverage, LOH not overlapping
# with copy loss segments, smooth&merge segments within 300kb and prune
# segments smaller than 300kb.
segs.clean <- trim_to_coverage(segments, oncoscan_na33.cov) %>%
    adjust_loh() %>%
    merge_segments() %>%
    prune_by_size()
```

Of note, the `oncoscan_na33.cov` objects contains the genomic coverage of the 
oncoscan assay (start/end for each chromosomal arm, hg19). One could re-compute 
the latter by downloading the annotation file from the ThermoFisher website
and process it with the `get_oncoscan_coverage_from_probes`function.

A similar function is available for loading a result file from the ASCAT 
program: `load_ascat`. The ASCAT file is expected to have the following column 
names:
- 'chr' (chromosome number, with or withour "chr")
- 'startpos' (first position of CNV segment)
- 'endpos' (last position of CNV segment)
- 'nMajor' (Number of copies of the major allele)
- 'nMinor' (Number of copies of the minor allele)

```{r}
ascat.fn <- system.file("extdata", "ascat_example.txt", package = "oncoscanR")
ascat.segments <- load_ascat(ascat.fn, oncoscan_na33.cov)
head(ascat.segments)
```

## Computation of arm-level alteration
Function `armlevel_alt`

An arm is declared globally altered if more than 80% of its bases are altered 
with a similar CNV type (amplifications [3 extra copies or more], gains [1-2 
extra copies], losses or copy-neutral losses of heterozygozity [LOH]). For 
instance, "gain of 3p" indicates that there is more than 80% of arm with 3 
copies but less than 80% with 5 (otherwise it would be an amplification). Prior 
to computation, segments of same copy number and at a distance <300Kbp (Oncoscan
resolution genome-wide) are merged. The remaining segments are filtered to a 
minimum size of 300Kbp.

For instance if we want to get all arms that have a global LOH alteration, we 
run:
```{r}
chas.fn <- system.file("extdata", "triploide_gene_list_full_location.txt", 
                       package = "oncoscanR")
segments <- load_chas(chas.fn, oncoscan_na33.cov)
armlevel.loh <- get_loh_segments(segments) %>% 
    armlevel_alt(kit.coverage = oncoscan_na33.cov)
```

The variable `armlevel.loh`is a named vector containing the arms that have 
percentage of base with LOH above the threshold (90%). To obtain the percentage 
of LOH bases in all arms, one could set the threshold to zero:

```{r}
armlevel.loh <- get_loh_segments(segments) %>%
        armlevel_alt(kit.coverage = oncoscan_na33.cov, threshold = 0)
```


## Global level of alteration
Several functions are available to perform such computation:

- `score_avgcn`: compute the average copy number across the genome
- `score_estwgd`: computes an estimation of the number of whole-genome doubling 
events
- `score_mbalt`: computes the total number of Mbp that have an alteration (w/o 
LOH segments)

```{r}
mbalt <- score_mbalt(segments, oncoscan_na33.cov)
percent.alt <- mbalt['sample']/mbalt['kit']
message(paste(mbalt['sample'], 'Mbp altered ->', round(percent.alt*100), 
              '% of genome'))

avgcn <- score_avgcn(segments, oncoscan_na33.cov)
wgd <- score_estwgd(segments, oncoscan_na33.cov)
message(paste('Average copy number:', round(avgcn, 2), '->', wgd['WGD'], 
'whole-genome doubling event'))
```


## HRD scores
The package contains several HRD scores described below.

### Score LST 
Function `score_lst`

Procedure based on the paper from Popova et al, Can. Res. 2012 (PMID: 22933060).
First segments smaller than 3Mb are removed, then segments are smoothed with 
respect to copy number at a distance of 3Mb.
The number of LSTs is the number of breakpoints (breakpoints closer than 3Mb are
merged) that have a segment larger or equal to 10Mb on each side. This score was
linked to BRCA1/2-deficient tumors.

### Score HR-LOH 
Function `score_loh`

Procedure based on the paper from Abkevich et al., Br J Cancer 2012 (PMID: 
23047548). 
Number of LOH segments larger than 15Mb but excluding segments on chromosomes 
with a global LOH alteration. This score was linked to BRCA1/2-deficient tumors.

### Score nLST
Function `score_nlst`

HRD score developed at HUG and based on the LST score by Popova et al. but 
normalized by an estimation of the number of whole-genome doubling events.
Of note, copy-neutral LOH segments are removed before computation.

`nLST = LST - 7*W/2` where `W` is the number of whole-genome doubling events.

The score is positive if there are at least 15 nLST.

The nLST score has been validated on 469 high grade ovarian cancer samples from
the PAOLA-1 clinical trial and is used in routine at the Geneva University 
Hospitals for prediction of PARP inhibitors response.

*How to cite*

Christinat Y, Ho L, Clément S, et al. 2022-RA-567-ESGO The Geneva HRD test: 
clinical validation on 469 samples from the PAOLA-1 trial. International Journal
of Gynecologic Cancer 2022;32:A238-A239.


### Score gLOH
Function `score_gloh`

The percentage genomic LOH score is computed as described in the FoundationFocus
CDx BRCA LOH assay; i.e. the percentage of bases covered by the Oncoscan that 
display a loss of heterozygosity independently of the number of copies, 
excluding chromosomal arms that have a global LOH (>=90% of arm length). To 
compute with the armlevel_alt function on LOH segments only). This score was 
linked to BRCA1/2-deficient tumors.


### Example
First we need to load and clean the ChAS export file (from a female patient). We
adjust the  Oncoscan coverage to exclude the 21p arm as it is only partially 
covered.

```{r}
# Load data
chas.fn <- system.file("extdata", "LST_gene_list_full_location.txt", 
                       package = "oncoscanR")
segments <- load_chas(chas.fn, oncoscan_na33.cov)

# Clean the segments: restricted to Oncoscan coverage, LOH not overlapping
# with copy loss segments, smooth&merge segments within 300kb and prune
# segments smaller than 300kb.
segs.clean <- trim_to_coverage(segments, oncoscan_na33.cov) %>%
    adjust_loh() %>%
    merge_segments() %>%
    prune_by_size()

# Then we need to compute the arm-level alteration for loss and LOH since many 
# scores discard arms that are globally altered.
arms.loss <- names(get_loss_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscan_na33.cov))
arms.loh <- names(get_loh_segments(segs.clean) %>%
        armlevel_alt(kit.coverage = oncoscan_na33.cov))

# Get the number of LST
lst <- score_lst(segs.clean, oncoscan_na33.cov)

# Get the number of HR-LOH
hrloh <- score_loh(segs.clean, arms.loh, arms.loss, oncoscan_na33.cov)

# Get the genomic LOH score
gloh <- score_gloh(segs.clean, arms.loh, arms.loss, oncoscan_na33.cov)

# Get the number of nLST
wgd <- score_estwgd(segs.clean, oncoscan_na33.cov)  # Get the avg CN, including 21p
nlst <- score_nlst(segs.clean, wgd["WGD"], oncoscan_na33.cov)

print(c(LST=lst, `HR-LOH`=hrloh, gLOH=gloh, nLST=nlst))
```


## TDplus score
function `score_td`

Procedure based on the paper from Popova et al., Cancer Res 2016 (PMID: 
26787835). The TDplus score is defined as the number of regions larger than 1Mb 
but smaller or equal to 10Mb with a gain of one or two copies. This score was 
linked to CDK12-deficient tumors. 
They also identified as second category of tandem duplication whose size is 
smaller or equal than 1Mb and around 300Kb but could not link it to a phenotype.
Note that due to its resolution the Oncoscan assay will most likely miss  this 
second category. Nonetheless it is reported by the function but not by the 
standard workflow.

```{r}
# Load data
chas.fn <- system.file("extdata", "TDplus_gene_list_full_location.txt", 
                       package = "oncoscanR")
segments <- load_chas(chas.fn, oncoscan_na33.cov)

# Clean the segments: restricted to Oncoscan coverage, LOH not overlapping
# with copy loss segments, smooth&merge segments within 300kb and prune
# segments smaller than 300kb.
segs.clean <- trim_to_coverage(segments, oncoscan_na33.cov) %>%
    adjust_loh() %>%
    merge_segments() %>%
    prune_by_size()

td <- score_td(segs.clean)
print(td$TDplus)
```


## Main workflow (as used at the Geneva University Hospitals)
The main workflow used for routine analysis can be launched either in R via the 
`workflow_oncoscan.chas(chas.fn, gender)` function or via the script 
`bin/run_oncoscan_workflow.R`:

Usage: 
`Rscript path_to_oncoscanR_package/bin/oncoscan-workflow.R CHAS_FILE`
- `CHAS_FILE`: Path to the text export file from ChAS or a compatible text file.

The script will output a JSON string into the terminal with all the computed 
information. :

```
{
  "armlevel": {
    "AMP": [],
    "LOSS": ["17p", "2q", "4p"],
    "LOH": ["14q", "5q", "8p", "8q"],
    "GAIN": [19p", "19q", "1q", "20p", "20q", "3q", "5p", "6p", "9p", "9q", 
    "Xp", "Xq"]
  },
  "scores": {
    "HRD": "Negative, nLST=12",
    "TDplus": 22,
    "avgCN": "2.43"
  },
  "file": "H19001012_gene_list_full_location.txt"
}
```

Or to launch the workflow within R:
```{r}
segs.filename <- system.file('extdata', 'LST_gene_list_full_location.txt', 
                             package = 'oncoscanR')
dat <- workflow_oncoscan.chas(segs.filename)

message(paste('Arms with copy loss:', 
              paste(dat$armlevel$LOSS, collapse = ', ')))
message(paste('Arms with copy gains:', 
              paste(dat$armlevel$GAIN, collapse = ', ')))
message(paste('HRD score:', dat$scores$HRD))
```

A similar function is available for running the workflow from an ASCAT result
file: `workflow_oncoscan.ascat`.

```{r}
library(jsonlite)
segs.filename <- system.file('extdata', 'ascat_example.txt', 
                             package = 'oncoscanR')
dat <- workflow_oncoscan.ascat(segs.filename)
toJSON(dat, auto_unbox=TRUE, pretty=TRUE)
```


Please read the manual for a description of all available R functions.

## References
1. "Homologous Recombination Deficiency (HRD) Score Predicts Response to 
Platinum-Containing Neoadjuvant Chemotherapy in Patients with Triple-Negative 
Breast Cancer.", M. Telli et al., Clin Cancer Res volume 22(15), august 2016.
2. "Ovarian Cancers Harboring Inactivating Mutations in CDK12 Display a Distinct
Genomic Instability Pattern Characterized by Large Tandem Duplications.", T. 
Popova et al., Cancer Res volume 76(7), april 2016.
3. "Ploidy and large-scale genomic instability consistently identify basal-like 
breast carcinomas with BRCA1/2 inactivation.", T. Popova et al., Cancer Res 
volume 72(21), november 2012.
4. "Patterns of genomic loss of heterozygosity predict homologous recombination 
repair defects in epithelial ovarian cancer.", V. Abkevich et al., Br J Cancer. 
2012 Nov 6;107(10).
5. "Absolute quantification of somatic DNA alterations in human cancer", S. 
Carter et al., Nat Biotech, 2012 volume 30(5).


## Session info
Here is the output of `sessionInfo()` on the system on which this document was 
compiled:

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{load_ascat}
\alias{load_ascat}
\title{Load am ASCAT text export file.}
\usage{
load_ascat(filename, kit.coverage)
}
\arguments{
\item{filename}{Path to the ASCAT file.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm by the kit.}
}
\value{
A \code{GRanges} object containing the segments, their copy number
(field \code{cn}), their copy
number types (field \code{cntype}). \code{cntype} contains either 'Gain',
'Loss' or 'LOH'.
If the file contains twice the same segment or does not respect the format
specifications, then an error is raised. 
NB. If the chromosome name is in the format '1' and not 'chr1' and will
be transformed if needed.
}
\description{
Load am ASCAT text export file.
}
\details{
The ASCAT file is expected to have the following column names:
'chr' (chromosome number), 'startpos' (first position of CNV segment),
'endpos' (last position of CNV segment), 'nMajor' (Number of copies of the 
major allele) and 'nMinor' (Number of copies of the minor allele).

The segments are attributed to each chromosome arm and split if necessary.
}
\examples{
segs.filename <- system.file('extdata', 'ascat_example.txt',
  package = 'oncoscanR')
segs.ascat_example <- load_ascat(segs.filename, oncoscan_na33.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/workflow_oncoscan.R
\name{workflow_oncoscan.chas}
\alias{workflow_oncoscan.chas}
\title{Run the standard workflow for Oncoscan ChAS files.}
\usage{
workflow_oncoscan.chas(chas.fn)
}
\arguments{
\item{chas.fn}{Path to the text-export ChAS file}
}
\value{
A list of lists with the following elements:
\code{armlevel = list(AMP= list of arms, GAIN= list of arms, LOSS= list of
arms, LOH= list of arms),
scores = list(LST= number, LOH= number, TDplus= number, TD= number),
file = path of the ChAS file as given by the parameter)}
}
\description{
Run the standard workflow for Oncoscan ChAS files.
}
\details{
Identifies the globally altered arms (\>=90\% of arm altered),
computes the HRD and TD+ scores. The amplification is defined as a CN subtype
 \code{cntype.weakamp} or \code{cntype.strongamp}. An arm is gained if of CN
 type \code{cntype.gain} unless the arm is amplified.
}
\examples{
segs.filename <- system.file('extdata', 'chas_example.txt',
package = 'oncoscanR')
workflow_oncoscan.chas(segs.filename)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{prune_by_size}
\alias{prune_by_size}
\title{Remove segments smaller than the kit resolution.}
\usage{
prune_by_size(segments, threshold = 300)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{threshold}{Number indicating the minimum segment size to be kept (in
kilobases).
Defaults to the Oncoscan assay resolution outside of cancer genes: 300Kb.}
}
\value{
A \code{GRanges} object containing the cleaned segments, their copy
number and copy number types.
}
\description{
Remove segments smaller than the kit resolution.
}
\examples{
segs.300k <- prune_by_size(segs.chas_example)
segs.50k <- prune_by_size(segs.chas_example, 50)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scores.R
\name{score_estwgd}
\alias{score_estwgd}
\title{Estimates the number of whole-genome doubling events (WGD).}
\usage{
score_estwgd(segments, kit.coverage)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm.}
}
\value{
A named list with two values: WGD (whole-genome doubling events) and
avgCN (the average copy number). WGD values are 0 for no WGD event, 1 for one
 WGD event, 2 for several WGD events.
}
\description{
Estimates the number of whole-genome doubling events (WGD).
}
\details{
Based on the publication from Carter et al. (Nature Biotechnology
2012; PubMed ID: 22544022).
On a pan-cancer cohort, they observed that tumors that underwent one
whole-genome doubling event had a ploidy (average copy number) between 2.2
and 3.4. This function relies on the function \code{score_avgcn} to compute
the ploidy.
}
\examples{
score_estwgd(segs.chas_example, oncoscan_na33.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{trim_to_coverage}
\alias{trim_to_coverage}
\title{Trim segments with respect to the kit's coverage.}
\usage{
trim_to_coverage(segments, kit.coverage)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm.}
}
\value{
A \code{GRanges} object containing the cleaned segments, their copy
number and copy number types.
}
\description{
Trim segments with respect to the kit's coverage.
}
\details{
All segments that are not entirely contained within the kit coverage
 will be trimmed to the coverage's limits.
}
\examples{
segs.trimmed <- trim_to_coverage(segs.chas_example, oncoscan_na33.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{get_hetloss_segments}
\alias{get_hetloss_segments}
\title{Return all segments with heterozygous loss.}
\usage{
get_hetloss_segments(segments)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}
}
\value{
A \code{GRanges} object containing the selected segments, their copy
number and copy number types.
}
\description{
Return all segments with heterozygous loss.
}
\examples{
segs.hetloss <- get_hetloss_segments(segs.chas_example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scores.R
\name{score_mbalt}
\alias{score_mbalt}
\title{Computes the total number of Mbp altered.}
\usage{
score_mbalt(segments, kit.coverage, loh.rm = TRUE)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm.}

\item{loh.rm}{A boolean (TRUE by default) to indicate whether LOH segments
should be excluded.}
}
\value{
A named list representing the Mbp altered in the sample and the total
 Mbp of the kit.
}
\description{
Computes the total number of Mbp altered.
}
\examples{
score_mbalt(segs.chas_example, oncoscan_na33.cov)
score_mbalt(segs.chas_example, oncoscan_na33.cov, FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{adjust_loh}
\alias{adjust_loh}
\title{Trim LOH segments with respect to loss segments.}
\usage{
adjust_loh(segments)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}
}
\value{
A \code{GRanges} object containing the cleaned segments, their copy
number and copy number types.
}
\description{
Trim LOH segments with respect to loss segments.
}
\details{
LOH segments completely contained within (or equal to)  a copy loss
segment are deleted.
LOH segments partially overlapping (on one end only) with a copy loss segment
 are trimmed to remove the overlap or split into several segments.
}
\examples{
segs.adj <- adjust_loh(segs.chas_example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scores.R
\name{score_loh}
\alias{score_loh}
\title{Compute the number HR deficiency-associated LOH regions.}
\usage{
score_loh(segments, arms.loh, arms.hetloss, kit.coverage)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{arms.loh}{A list of arms with global/arm-level LOH alteration.}

\item{arms.hetloss}{A list of arms with global/arm-level heterozygous
losses.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm.}
}
\value{
An integer representing the number of HRD-LOH regions.
}
\description{
Compute the number HR deficiency-associated LOH regions.
}
\details{
Procedure based on the paper from Abkevich et al., Br J Cancer 2012
(PMID: 23047548). All LOH segments larger than 15Mb but excluding chromosome
with a global LOH alteration (to compute with the \code{armlevel_alt}
function on LOH segments only). This score was linked to BRCA1/2-deficient
tumors.
Note that the function will merge overlapping or neighbor LOH segments (at a
distance of 1bp).
}
\examples{
armlevel.loh <- armlevel_alt(get_loh_segments(segs.chas_example),
                             kit.coverage = oncoscan_na33.cov)
armlevel.hetloss <- armlevel_alt(get_hetloss_segments(segs.chas_example),
                             kit.coverage = oncoscan_na33.cov)
score_loh(segs.chas_example, names(armlevel.loh), names(armlevel.hetloss),
oncoscan_na33.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scores.R
\name{score_td}
\alias{score_td}
\title{Compute the number of large tandem duplication (TDplus).}
\usage{
score_td(segments)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}
}
\value{
A list of integer containing the TDplus score (\code{'TDplus'}) and
the small TD score (\code{'TD'}).
}
\description{
Compute the number of large tandem duplication (TDplus).
}
\details{
Procedure based on the paper from Popova et al., Cancer Res 2016
(PMID: 26787835). The TDplus score is defined as the number of regions larger
 than 1Mb but smaller or equal to 10Mb with a gain of one or two copies. This
 score was linked
 to CDK12-deficient tumors. They also identified as second category of tandem
  duplication whose size is smaller or equal than 1Mb and around 300Kb but
  could not link it to a phenotype. Note that due to its resolution the
  Oncoscan assay will most likely miss this second category. Nonetheless it
  is reported by the function.
}
\examples{
score_td(segs.chas_example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scores.R
\name{score_lst}
\alias{score_lst}
\title{Compute the number of Large-scale State Transitions (LSTs).}
\usage{
score_lst(segments, kit.coverage)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm.}
}
\value{
An integer representing the number of LSTs.
}
\description{
Compute the number of Large-scale State Transitions (LSTs).
}
\details{
Procedure based on the paper from Popova et al, Can. Res. 2012
(PMID: 22933060). First segments smaller than 3Mb are removed, then segments
are smoothed with respect to copy number at a distance of 3Mb.
The number of LSTs is the number of breakpoints (breakpoints closer than 3Mb
are merged) that have a segment larger or equal to 10Mb on each side. This
score was linked to BRCA1/2-deficient tumors.
}
\examples{
score_lst(segs.chas_example, oncoscan_na33.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/oncoscanR-package.R
\docType{package}
\name{oncoscanR-package}
\alias{oncoscanR}
\alias{oncoscanR-package}
\title{oncoscanR: an R package to perform secondary analysis of Copy Number 
Variation data.}
\description{
Allows computation of different homologous recombination deficiency (HRD) 
scores to identify PARP inhibitors responders. The package also allows for 
computation of the tandem duplication plus score (TDplus; hallmark of 
CDK12-mutated tumors) and the identification of arm-level alterations (e.g. 
gain of chromosome arm 1p).
}
\details{
The package is tailored for the ThermoFisher Oncoscan assay analyzed with 
their Chromosome Alteration Suite (ChAS) but can be adapted to any input.

To learn more about \code{oncoscanR}, see the vignette using 
\code{browseVignettes(package = "oncoscanR")}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/yannchristinat/oncoscanR}
  \item Report bugs at \url{https://github.com/yannchristinat/oncoscanR/issues}
}

}
\author{
\strong{Maintainer}: Yann Christinat \email{yann.christinat@hcuge.ch}

Authors:
\itemize{
  \item Geneva University Hospitals [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scores.R
\name{score_gloh}
\alias{score_gloh}
\title{Compute the genomic LOH score.}
\usage{
score_gloh(segments, arms.loh, arms.hetloss, kit.coverage)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{arms.loh}{A list of arms with global/arm-level LOH alteration.}

\item{arms.hetloss}{A list of arms with global/arm-level heterozygous
loss.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm.}
}
\value{
An integer representing the percentage of LOH bases.
}
\description{
Compute the genomic LOH score.
}
\details{
The percentage genomic LOH score is computed as described in the
FoundationFocus CDx BRCA LOH assay; i.e. the percentage of bases covered by
the Oncoscan that display a loss of heterozygosity independently of the
number of copies, excluding chromosomal arms that have a global LOH (>=90% of
arm length).
To compute with the \code{armlevel_alt} function on LOH segments only).
This score was linked to BRCA1/2-deficient tumors.
}
\examples{
armlevel.loh <- armlevel_alt(get_loh_segments(segs.chas_example),
                             kit.coverage = oncoscan_na33.cov)
armlevel.hetloss <- armlevel_alt(get_hetloss_segments(segs.chas_example),
                             kit.coverage = oncoscan_na33.cov)
score_gloh(segs.chas_example, names(armlevel.loh), names(armlevel.hetloss),
oncoscan_na33.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{get_gain_segments}
\alias{get_gain_segments}
\title{Return all segments with gain of copies.}
\usage{
get_gain_segments(segments)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}
}
\value{
A \code{GRanges} object containing the selected segments, their copy
number and copy number types.
}
\description{
Return all segments with gain of copies.
}
\examples{
segs.gain <- get_gain_segments(segs.chas_example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{segs.chas_example}
\alias{segs.chas_example}
\title{Expected segments from loading the ChAS file 'chas_example.txt'.}
\format{
A \code{GRanges} object containing the segments, their copy number
(field \code{cn}) and their copy number types (field \code{cn.type}).
}
\source{
segs.filename <- system.file('extdata', 'chas_example.txt', 
package = 'oncoscanR')
mykit.cov <- get_oncoscan_coverage_from_probes()
segs.chas_example <- load_chas(segs.filename, kit.coverage = mykit.cov)
}
\usage{
segs.chas_example
}
\description{
Expected segments from loading the ChAS file 'chas_example.txt'.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{load_chas}
\alias{load_chas}
\title{Load a ChAS text export file.}
\usage{
load_chas(filename, kit.coverage)
}
\arguments{
\item{filename}{Path to the ChAS file.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm by the kit.}
}
\value{
A \code{GRanges} object containing the segments, their copy number
(field \code{cn}), their copy
number types (field \code{cntype}). \code{cntype} contains either 'Gain',
'Loss' or 'LOH'.
If the file contains twice the same segment or does not respect the format
specifications, then an error is
raised. NB. The chromosome name is in the format '1' and not 'chr1' and will
be transformed if needed.
}
\description{
Load a ChAS text export file.
}
\details{
The ChAS file is expected to have the following column names:
'CN State' (number or empty), 'Type' (expected value: 'Gain', 'Loss' or
'LOH') and 'Full Location' (in the format 'chr:start-end').

The segments are attributed to each chromosome arm and split if necessary.
}
\examples{
segs.filename <- system.file('extdata', 'chas_example.txt',
  package = 'oncoscanR')
segs.chas_example <- load_chas(segs.filename, oncoscan_na33.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{get_oncoscan_coverage_from_bed}
\alias{get_oncoscan_coverage_from_bed}
\title{Load the oncoscan coverage BED file into a GenomicRanges object.}
\usage{
get_oncoscan_coverage_from_bed(filename)
}
\arguments{
\item{filename}{Path to the coverage BED file.}
}
\value{
A \code{GRanges} object containing the regions covered on each
chromosome arm.
}
\description{
Load the oncoscan coverage BED file into a GenomicRanges object.
}
\details{
Expects the following columns from the BED file (no header):
  1. Name of the chromosomal arm (e.g. "1p")
  2. Start position of the arm
  3. End position of the arm
}
\examples{
oncoscan_na33.cov <- get_oncoscan_coverage_from_bed(
       system.file('extdata', 'Oncoscan.na33.r2.cov.processed.bed',
       package = 'oncoscanR'))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{cntypes}
\alias{cntypes}
\title{Accepted types of CN for the segments
  - 'Gain': 1-2 extra copies
  - 'Weak amplification': 3-7 extra copies
  - 'Strong amplification': 8 or more extra copies
  - 'Heterozygote loss': Loss of one copy out of two
  - 'Homozygote loss': Loss of all copies
  - 'LOH': copy-neutral loss of one parental allele}
\format{
A list of strings containing all CN types
}
\source{
cntypes <- list(LOH='LOH', Gain='Gain', Loss='Loss')
}
\usage{
cntypes
}
\description{
Accepted types of CN for the segments
  - 'Gain': 1-2 extra copies
  - 'Weak amplification': 3-7 extra copies
  - 'Strong amplification': 8 or more extra copies
  - 'Heterozygote loss': Loss of one copy out of two
  - 'Homozygote loss': Loss of all copies
  - 'LOH': copy-neutral loss of one parental allele
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{get_loh_segments}
\alias{get_loh_segments}
\title{Return all segments of type LOH, independently of the copy number.}
\usage{
get_loh_segments(segments)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}
}
\value{
A \code{GRanges} object containing the selected segments, their copy
number and copy number types.
}
\description{
Return all segments of type LOH, independently of the copy number.
}
\examples{
segs.loh <- get_loh_segments(segs.chas_example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{oncoscan_na33.cov}
\alias{oncoscan_na33.cov}
\title{GenomicRanges object of the chromosomal arms coverage for the oncoscan assay
(based on file extdata/Oncoscan.na33.r2.cov.processed.bed).}
\format{
A \code{GRanges} object containing the region covered on each
chromosome arm.
}
\source{
\code{oncoscan_na33.cov <- get_oncoscan_coverage_from_bed(
    system.file('extdata', 'Oncoscan.na33.r2.cov.processed.bed',
                package = 'oncoscanR'))}
}
\usage{
oncoscan_na33.cov
}
\description{
GenomicRanges object of the chromosomal arms coverage for the oncoscan assay
(based on file extdata/Oncoscan.na33.r2.cov.processed.bed).
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{merge_segments}
\alias{merge_segments}
\title{Merge segments with respect to the kit resolution and the copy number.}
\usage{
merge_segments(segments, kit.resolution = 300)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{kit.resolution}{Number >0 indicating the minimum segment size
detectable by the technique (in kilobases).
Defaults to the Oncoscan assay resolution outside of cancer genes: 300Kb.}
}
\value{
A \code{GRanges} object containing the cleaned segments, their copy
number and copy number types.
}
\description{
Merge segments with respect to the kit resolution and the copy number.
}
\details{
If two segments are at a distance smaller than the resolution, then
the segments are merged if the
share the same \code{cn} value. Note that the function does not look at the
copy number type or subtype but
only at the actual copy number to decide whether segments can be merged.
}
\examples{
segs.merged <- merge_segments(segs.chas_example)
segs.merged_50k <- merge_segments(segs.chas_example, 50)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{get_homloss_segments}
\alias{get_homloss_segments}
\title{Return all segments with homozygous loss.}
\usage{
get_homloss_segments(segments)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}
}
\value{
A \code{GRanges} object containing the selected segments, their copy
number and copy number types.
}
\description{
Return all segments with homozygous loss.
}
\examples{
get_homloss_segments <- get_homloss_segments(segs.chas_example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scores.R
\name{armlevel_alt}
\alias{armlevel_alt}
\title{Get all globally-altered chromosome arms.}
\usage{
armlevel_alt(segments, kit.coverage, threshold = 0.9)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm.}

\item{threshold}{The minimum percentage of the arm to be considered as
globally altered. Defaults to 80\%.}
}
\value{
A list of globally-altered chromosome arms with the percentage of arm
 altered.
}
\description{
Get all globally-altered chromosome arms.
}
\details{
By default uses the sum of all alterations and set the arm as
globally altered if \>80\% of the arm is altered. Does not account for
alteration type and copy number.
Will run the function \code{trim_to_coverage} on the segments.
}
\examples{
arms <- armlevel_alt(segs.chas_example, oncoscan_na33.cov, 0.9)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scores.R
\name{score_nlst}
\alias{score_nlst}
\title{Compute the number of LSTs, normalized by the number of WGD events.}
\usage{
score_nlst(segments, n.wgd, kit.coverage, threshold = 15)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{n.wgd}{Number of whole-genome doubling events (0 if diploid).}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm.}

\item{threshold}{A number above which the test is returned positive (>=).}
}
\value{
A named list with the number of nLSTs and the corresponding label
('Positive', 'Negative').
}
\description{
Compute the number of LSTs, normalized by the number of WGD events.
}
\details{
Compute the number of LSTs in non-LOH segments via the
\code{score_lst} function and subtract the extra noise induced by WGD events:
 nLST = LST - 7*W/2 where W is the number of WGD events.
A sample is HRD positive (deficient in HR pathway) if nLST is greater or
equal to the threshold (15 by default).
This score was linked to BRCA1/2-deficient tumors.
}
\examples{
w <- score_estwgd(segs.chas_example, oncoscan_na33.cov)
score_nlst(segs.chas_example, w['WGD'], oncoscan_na33.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scores.R
\name{score_avgcn}
\alias{score_avgcn}
\title{Compute the average copy number variation across the genome.}
\usage{
score_avgcn(segments, kit.coverage)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}

\item{kit.coverage}{A \code{GRanges} object containing the regions covered on
each chromosome arm.}
}
\value{
A decimal value
}
\description{
Compute the average copy number variation across the genome.
}
\details{
Compute the weighted average (by segment length) of the copy number
variation. LOH segments and sexual chromosomes are excluded. Copy number
variation is rounded to the next level (1.67 -> 1 but 2.33 -> 3).
}
\examples{
score_avgcn(segs.chas_example, oncoscan_na33.cov)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/workflow_oncoscan.R
\name{workflow_oncoscan.ascat}
\alias{workflow_oncoscan.ascat}
\title{Run the standard workflow for ASCAT files (from oncoscan data).}
\usage{
workflow_oncoscan.ascat(ascat.fn)
}
\arguments{
\item{ascat.fn}{Path to the text-export ASCAT file}
}
\value{
A list of lists with the following elements:
\code{armlevel = list(AMP= list of arms, GAIN= list of arms, LOSS= list of
arms, LOH= list of arms),
scores = list(LST= number, LOH= number, TDplus= number, TD= number),
file = path of the ChAS file as given by the parameter)}
}
\description{
Run the standard workflow for ASCAT files (from oncoscan data).
}
\details{
Identifies the globally altered arms (\>=90\% of arm altered),
computes the HRD and TD+ scores. The amplification is defined as a CN>=5. 
An arm is gained if of CN type \code{cntype.gain} unless the arm is 
amplified.
}
\examples{
segs.filename <- system.file('extdata', 'ascat_example.txt',
package = 'oncoscanR')
workflow_oncoscan.ascat(segs.filename)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{get_amp_segments}
\alias{get_amp_segments}
\title{Return all segments with an amplification (5 or more copies).}
\usage{
get_amp_segments(segments)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}
}
\value{
A \code{GRanges} object containing the selected segments, their copy
number and copy number types.
}
\description{
Return all segments with an amplification (5 or more copies).
}
\examples{
segs.amp <- get_amp_segments(segs.chas_example)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/input.R
\name{get_loss_segments}
\alias{get_loss_segments}
\title{Return all segments with loss of 1 or 2 copies.}
\usage{
get_loss_segments(segments)
}
\arguments{
\item{segments}{A \code{GRanges} object containing the segments, their copy
number and copy number types.}
}
\value{
A \code{GRanges} object containing the selected segments, their copy
number and copy number types.
}
\description{
Return all segments with loss of 1 or 2 copies.
}
\examples{
segs.loh <- get_loh_segments(segs.chas_example)
}
